"""run all tests"""

import unittest
import glob
import sys

if __name__ == '__main__':
    if len(sys.argv)>1:
        #1. add LIMIX path to environment
        limix_path = sys.argv[1]
        sys.path.insert(0,limix_path)

    if len(sys.argv)>2:
	folders = [sys.argv[2]]
    else:
    	folders = set(glob.glob('*'))-set(glob.glob('*.*'))

    # Gather all tests in suite
    suite = unittest.TestSuite()
    for folder in folders:
        tests = unittest.TestLoader().discover(folder,'test*')
        suite.addTests(tests)

    # run all tests
    unittest.TextTestRunner(verbosity=2).run(suite)



"""LMM testing code"""
import unittest
import scipy as sp
import numpy as np
from limix.core.covar import LowRankCov
from limix.utils.check_grad import mcheck_grad

class TestLowRank(unittest.TestCase):
    """test class for CLowRankCF"""
    def setUp(self):
        sp.random.seed(1)
        self.n = 4
        self.rank = 2
        self.C = LowRankCov(self.n,self.rank)
        self.name = 'lowrank'
        self.C.setRandomParams()

    def test_grad(self):
        def func(x, i):
            self.C.setParams(x)
            return self.C.K()

        def grad(x, i):
            self.C.setParams(x)
            return self.C.K_grad_i(i)

        x0 = self.C.getParams()
        err = mcheck_grad(func, grad, x0)

        np.testing.assert_almost_equal(err, 0., decimal = 6)

    def test_param_activation(self):
        self.assertEqual(len(self.C.getParams()), 8)
        self.C.act_X = False
        self.assertEqual(len(self.C.getParams()), 0)

        self.C.setParams(np.array([]))
        with self.assertRaises(ValueError):
            self.C.setParams(np.array([0]))

        with self.assertRaises(ValueError):
            self.C.K_grad_i(0)

if __name__ == '__main__':
    unittest.main()



import sys
from limix.core.type.observed import Observed
from limix.core.type.cached import Cached, cached
from limix.utils.eigen import *
import scipy as sp
import scipy.linalg as LA
import scipy.sparse.linalg as sla
import warnings

import logging as LG

class Covariance(Cached, Observed):
    """
    abstract super class for all implementations of covariance functions
    """
    def __init__(self, dim=None, nIterMC=30):
        Cached.__init__(self)
        self._nIterMC = nIterMC
        self._reuse = True
        self._KiZo = None
        self._tol = 1e-6
        if dim is not None:
            self.initialize(dim)

    def initialize(self,dim):
        self.dim = dim
        self._use_to_predict = False

    def clear_all(self):
        self.clear_cache('covar_base')
        self._notify()

    #######################
    # Param Handling
    #######################

    def setRandomParams(self):
        """
        set random hyperparameters
        """
        params = sp.randn(self.getNumberParams())
        self.setParams(params)

    def setCovariance(self,cov):
        """
        set hyperparameters from given covariance
        """
        warnings.warn('not implemented')


    def perturbParams(self, pertSize=1e-3):
        """
        slightly perturbs the values of the parameters
        """
        params = self.getParams()
        self.setParams(params+pertSize*sp.randn(params.shape[0]))

    def _calcNumberParams(self):
        """
        calculates the number of parameters
        """
        warnings.warn('not implemented')

    ###################################
    # Non-cached methods
    ###################################
    def dot(self, M):
        return sp.dot(self.K(), M)

    def solve(self,M):
        return LA.cho_solve((self.chol(),True),M)

    def solve_ls(self, M, M0=None):
        if M0 is None:    M0 = 1E-3 * sp.randn(*M.shape)
        def veKvei(m):
            _M = m.reshape(M.shape, order='F')
            return self.dot(_M).reshape(M.size, order='F')
        Kx_O = sla.LinearOperator((M.size, M.size), matvec=veKvei, rmatvec=veKvei, dtype='float64')
        # vectorize
        m  = M.reshape(M.size, order='F')
        m0 = M0.reshape(M0.size, order='F')
        r, _ = sla.cgs(Kx_O, m, x0=m0, tol=self._tol)
        return r.reshape(M.shape, order='F')

    ####################################
    # cached
    ####################################
    @cached('covar_base')
    def K(self):
        """
        evaluates the kernel for given hyperparameters theta and inputs X
        """
        LG.critical("implement K")
        print("%s: Function K not yet implemented"%(self.__class__))
        return None

    @cached('covar_base')
    def K_grad_i(self,i):
        """
        partial derivative with repspect to the i-th hyperparamter theta[i]
        """
        LG.critical("implement K_grad_i")
        print("%s: Function K not yet implemented"%(self.__class__))

    @cached('covar_base')
    def K_hess_i_j(self,i,j):
        LG.critical("implement K_hess_i_j")
        print("%s: Function Khess not yet implemented"%(self.__class__))

    @cached('covar_base')
    def chol(self):
        return LA.cholesky(self.K()).T

    @cached('covar_base')
    def inv(self):
        return self.solve(sp.eye(self.dim))

    @cached('covar_base')
    def logdet(self):
        return 2*sp.log(sp.diag(self.chol())).sum()

    @cached('covar_base')
    def logdet_grad_i(self,i):
        return self.solve(self.K_grad_i(i)).diagonal().sum()

    @cached('covar_base')
    def S(self):
        RV,U = LA.eigh(self.K())
        self.fill_cache('U',U)
        return RV

    @cached('covar_base')
    def U(self):
        S,RV = LA.eigh(self.K())
        self.fill_cache('S',S)
        return RV

    @cached('covar_base')
    def USi2(self):
        # U * S**(-1/2)
        return self.U()*(self.S()**(-0.5))

    ###########################
    # The following methods are deprecated
    ############################

    #@cached
    #def Sgrad(self,i):
    #    return dS_dti(self.Kgrad_param(i),U=self.U())

    #@cached
    #def Ugrad(self,i):
    #    return dU_dti(self.Kgrad_param(i),U=self.U(),S=self.S())

    #@cached
    #def USi2grad(self,i):
    #    # dU * S**(-1/2) + U * (-1/2 S**(-3/2) dS)
    #    Si2grad = -0.5*self.S()**(-1.5)*self.Sgrad(i)
    #    return self.Ugrad(i)*(self.S()**(-0.5)) + self.U()*Si2grad

    ###########################
    # Monte Carlo methods
    ###########################
    def resample(self):
        self.clear_cache('Z')
        self._notify()

    @cached('Z')
    def Z(self):
        r = sp.randn(self.dim, self._nIterMC)
        # norm Z to improve convergence
        norm = sp.sqrt(self.dim / (float(self._nIterMC) * (r**2).sum(0)))
        return norm * r

    @cached(['covar_base', 'Z'])
    def DKZ(self):
        R = sp.zeros((self.dim, self._nIterMC, self.getNumberParams()))
        for i in range(R.shape[2]):
            R[:, :, i] = sp.dot(self.K_grad_i(i), self.Z())
        return R 

    @cached(['covar_base', 'Z'])
    def DDKZ(self):
        R = sp.zeros((self.dim, self._nIterMC, self.getNumberParams(), self.getNumberParams()))
        for i in range(R.shape[2]):
            R[:, :, i, i] = sp.dot(self.K_hess_i_j(i, i), self.Z())
            for j in range(i):
                R[:, :, i, j] = sp.dot(self.K_hess_i_j(i, j), self.Z())
                R[:, :, j, i] = R[:, :, i, j]
        return R 

    @cached(['covar_base', 'Z'])
    def KiZ(self):
        R = self.solve_ls(self.Z(), M0=self._KiZo)
        if self._reuse:     self._KiZo = R
        return R
        
    @cached(['covar_base', 'Z'])
    def sample_logdet_grad_i(self, i):
        DiKZ = sp.dot(self.K_grad_i(i), self.Z())
        return (DiKZ * self.KiZ()).sum()

    @cached(['covar_base', 'Z'])
    def sample_logdet_grad(self):
        return (self.DKZ() * self.KiZ()[:, :, sp.newaxis]).sum(axis=(0, 1))

    @cached(['covar_base', 'Z'])
    def sample_trKiDDK(self):
        return (self.DDKZ() * self.KiZ()[:, :, sp.newaxis, sp.newaxis]).sum(axis=(0, 1))


    ###########################
    # Predictions
    ###########################
    @property
    def use_to_predict(self):
        return self._use_to_predict

    @use_to_predict.setter
    def use_to_predict(self,value):
        LG.critical("implement use_to_predict")
        print("%s: Function use_to_predict not yet implemented"%(self.__class__))

    @cached
    def Kcross(self):
        LG.critical("implement Kcross")
        print("%s: Function Kcross not yet implemented"%(self.__class__))

    ####################
    # Interpretable Params, Fisher information Matrix, std errors
    ####################
    def getInterParams(self):
        return self.getParams()

    def K_grad_interParam_i(self,i):
        LG.critical("implement K_grad_interParam_i")
        print("%s: Function K_grad_interParam_i not yet implemented"%(self.__class__))

    def getFisherInf(self):
        n_params = self.getNumberParams()
        R = sp.zeros((n_params,n_params))
        for m in range(n_params):
            for n in range(n_params):
                DnK = self.K_grad_interParam_i(m)
                DmK = self.K_grad_interParam_i(n)
                KiDnK = self.solve(DnK)
                KiDmK = self.solve(DmK)
                R[m,n] = 0.5*(KiDnK*KiDmK).sum()
        return R

    def setFIinv(self, value):
        self._FIinv = value

    def getFIinv(self):
        return self._FIinv

    ############################
    # Debugging
    ############################

    def Kgrad_param_num(self,i,h=1e-4):
        """
        check discrepancies between numerical and analytical gradients
        """
        params = self.getParams()
        e = sp.zeros_like(params); e[i] = 1
        self.setParams(params-h*e)
        C_L = self.K()
        self.setParams(params+h*e)
        C_R = self.K()
        self.setParams(params)
        RV = (C_R-C_L)/(2*h)
        return RV



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix
import limix.deprecated as dlimix
from limix.test import data
import os

class CIntearctLMM_test(unittest.TestCase):
    """
    test class for CIntearctLMM
    Status: currenlty only testing the special case of a CinteractLMM that is equilvanet to LMM
    """

    def setUp(self):
        self.datasets = ['lmm_data1']
        self.dir_name = os.path.dirname(os.path.realpath(__file__))

    def test_lmm(self):
        """basic main effec tests"""

        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            N = D['X'].shape[0]
            inter0 = SP.zeros([N,0])#fixed verion: all 0 feature did not work: #inter0 = SP.zeros([N,1])N-by-0 matrix instead of N-by-1 works
            inter1 = SP.ones([N,1])
            lmm = dlimix.CInteractLMM()
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            #inter0[:]=1
            #inter1[0:N/2]=-1
            lmm.setInter0(inter0)
            lmm.setInter(inter1)
            lmm.process()
            pv = lmm.getPv().ravel()
            D2= SP.sqrt( ((SP.log10(pv)-SP.log10(D['pv']))**2).mean())
            self.assertTrue(D2<1E-6)



    def test_permutation(self):
        #test permutation function
        for dn in self.datasets:
            D = data.load(os.path.join(self.dir_name,dn))
            perm = SP.random.permutation(D['X'].shape[0])
            N = D['X'].shape[0]
            inter0 = SP.zeros([N,0])#fix, as old version with all zero feature does not work#N-by-0 matrix instead of N-by-1 works
            inter1 = SP.ones([N,1])

            #1. set permuattion
            lmm = dlimix.CInteractLMM()
            lmm.setInter0(inter0)
            lmm.setInter(inter1)
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            if 1:
                #pdb.set_trace()
                perm = SP.array(perm,dtype='int32')#Windows needs int32 as long -> fix interface to accept int64 types
            lmm.setPermutation(perm)
            lmm.process()
            pv_perm1 = lmm.getPv().ravel()
            #2. do by hand
            lmm = dlimix.CInteractLMM()
            lmm.setInter0(inter0)
            lmm.setInter(inter1)
            lmm.setK(D['K'])
            lmm.setSNPs(D['X'][perm])
            lmm.setCovs(D['Cov'])
            lmm.setPheno(D['Y'])
            lmm.process()
            pv_perm2 = lmm.getPv().ravel()
            D2 = (SP.log10(pv_perm1)-SP.log10(pv_perm2))**2
            RV = SP.sqrt(D2.mean())
            self.assertTrue(RV<1E-6)


if __name__ == '__main__':
    unittest.main()



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix.deprecated as dlimix
from covar import Acovar_test

class CFreeFormCF_test(unittest.TestCase,Acovar_test):
    """test class for CFreeFormCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.C = dlimix.CFreeFormCF(self.n)
        self.name = 'CFreeFormCF'
        self.n_params=self.C.getNumberParams()
        K = self.C.K()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CRankOneCF_test(unittest.TestCase,Acovar_test):
    """test class for CRankOneCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.C = dlimix.CRankOneCF(self.n)
        self.name = 'CRankOneCF'
        self.n_params=self.C.getNumberParams()
        K = self.C.K()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CLowRankCF_test(unittest.TestCase,Acovar_test):
    """test class for CLowRankCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.rank=2
        self.C = dlimix.CLowRankCF(self.n,self.rank)
        self.name = 'CLowRankCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CDiagonalCF_test(unittest.TestCase,Acovar_test):
    """test class for CDiagonalCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=2
        self.C = dlimix.CDiagonalCF(self.n)
        self.name = 'CDiagonalCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CFixedCF_test(unittest.TestCase,Acovar_test):
    """test class for CFixedCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.C = dlimix.CFixedCF(SP.ones((self.n,self.n)))
        self.name = 'CFixedCF'
        self.n_params=self.C.getNumberParams()
        K = self.C.K()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CRank1diagCF_test(unittest.TestCase,Acovar_test):
    """test class for CRank1diagCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.C = dlimix.CRank1diagCF(self.n)
        self.name = 'CRank1diagCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CPolyCF_test(unittest.TestCase,Acovar_test):
    """test class for CRank1diagCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.d=2
        self.K=3
        self.C = dlimix.CPolyCF(self.n,self.d,self.K)
        self.name = 'CPolyCF'
        self.n_params=self.C.getNumberParams()
        params=SP.randn(self.n_params)
        self.C.setParams(params)

class CFixedDiagonalCF_test(unittest.TestCase,Acovar_test):
    """test class for CFixedDiagonalCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=4
        self.rank=1
        d = SP.rand(self.n)+1
        C0 = dlimix.CSumCF()
        C0.addCovariance(dlimix.CLowRankCF(self.n,self.rank))
        C0.addCovariance(dlimix.CDiagonalCF(self.n))
        self.C = dlimix.CFixedDiagonalCF(C0,d)
        self.name = 'CFixedDiagonalCF'
        self.n_params=self.C.getNumberParams()
        params=SP.randn(self.n_params)
        self.C.setParams(params)
        """
        h = 1e-4
        e = SP.zeros(self.n_params)
        for i in range(self.n_params):
            print i
            e[i] = 1
            self.C.setParams(params)
            Cgrad = self.C.Kgrad_param(i)
            self.C.setParams(params+h*e)
            Cr = self.C.K()
            self.C.setParams(params-h*e)
            Cl = self.C.K()
            Cgrad1 = (Cr-Cl)/(2*h)
            print Cgrad/Cgrad1
            pdb.set_trace()
            e[i] = 0
        """

if __name__ == '__main__':
    unittest.main()



import sys
from limix.core.type.cached import cached
import scipy as sp
from covar_base import Covariance
import pdb
import scipy.linalg as LA
import warnings

import logging as LG

class cov_reml(Covariance):
    """
    REML covariance used by GP for restricted maximum likelihood
    """
    def __init__(self,gp):
        Covariance.__init__(self)
        self.gp = gp
        gp.register(self.clear_all)
        self.dim = gp.mean.n_covs
        # self._calcNumberParams()

    #####################
    # Params handling
    #####################
    def setParams(self,params):
        warnings.warn('Read-only covariance type')

    def getParams(self,params):
        return self.gp.covar.getParams()

    # def _calcNumberParams(self):
    #     self.n_params = self.gp.covar.getNumberParams()

    #####################
    # Cached
    #####################
    @cached('covar_base')
    def K(self):
        return self.gp.Areml_K()

    @cached('covar_base')
    def K_grad_i(self,i):
        return self.gp.Areml_K_grad_i(i)



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import scipy as sp
import limix.deprecated.io.data_util as du
import pandas as pd

class genotype_reader_h5py():
    """
    a genotype reader using the HDF5 interface.
    """

    def __init__(self,file_name):
        self.file_name = file_name
        self.load()

    def load(self,cache_genotype=False,cache_phenotype=True):
        """load data file

        Args:
            cache_genotype:     load genotypes fully into memory (default: False)
            cache_phenotype:    load phentopyes fully intro memry (default: True)
        """
        import h5py
        self.f = h5py.File(self.file_name,'r')
        self.geno  = self.f['genotype']

        #parse out these we always need for convenience
        self.geno_matrix = self.geno['matrix']
        #dimensions
        self.num_samples = self.geno_matrix.shape[0]
        self.num_snps = self.geno_matrix.shape[1]

        self.sample_ID = self.geno['row_header']['sample_ID'][:]
        position   = {
            "chrom":    self.geno['col_header']['chrom'][:],
            "pos":      self.geno['col_header']['pos'][:],
            }
        if 'pos_cum' in self.geno['col_header'].keys():
            position['pos_cum']   = self.geno['col_header']['pos_cum'][:]

        if 'geno_ID' in self.geno['col_header'].keys():
            self.geno_ID   = self.geno['col_header']['geno_ID'][:]
        else:
            self.geno_ID =  sp.arange(self.num_snps)

        #cache?
        if cache_genotype:
            self.geno_matrix = self.geno_matrix[:]
        self.position=pd.DataFrame(data=position,index=self.geno_ID)


    def getGenotypes(self,sample_idx=None,idx_start=None,idx_end=None,chrom=None,pos_start=None,pos_end=None,center=True,unit=True,impute_missing=False,snp_idx=None,windowsize=0):
        """load genotypes.
        Optionally the indices for loading subgroups the genotypes for all people
        based on position of cumulative position.
        Positions can be given as (pos_start-pos_end on chrom)
        If both of these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            chrom:          chromosome based selection (return whole chromosome)
            pos_start:       position based selection (start position) tuple of chrom, position
            pos_end:       position based selection (end position) tuple of chrom, position
            impute_missing: Boolean indicator variable if missing values should be imputed

        Returns:
            X:          scipy.array of genotype values
        """
        #position based matching?
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None)):
            idx_start,idx_end=self.getGenoIndex(chrom=chrom,pos_start=pos_start,pos_end=pos_end,windowsize=windowsize)
        #index based matching?
        if (idx_start is not None) & (idx_end is not None):
            X = self.geno_matrix[:,idx_start:idx_end]
        elif snp_idx is not None:
            X = self.geno_matrix[:,snp_idx]
        else:
            X = self.geno_matrix[:,:]
        if sample_idx is not None:
            X=X[sample_idx]
        if impute_missing:
            X = du.imputeMissing(X,center=center,unit=unit)
        return X

    def getGenoIndex(self,pos_start=None,pos_end=None,windowsize=0):
        """computes 0-based genotype index from position of cumulative position.
        Positions can be given as (pos_start-pos_end on chrom)
        If both of these are None (default), then all genotypes are returned

        Args:
            chrom:          chromosome based selection (return whole chromosome)
            pos_start:      position based selection (start position) tuple of chrom, position
            pos_end:        position based selection (end position) tuple of chrom, position
            windowsize:     additionally include a flanking window around the selected positions (default 0)

        Returns:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
        """
        if (pos_start is not None) & (pos_end is not None):
            assert pos_start[0]==pos_end[0],   "getGenoIndex only supports selection on a single chromosome"
            I = self.position["chrom"]==pos_start[0]
            I = I & (self.postion["pos"]>=(pos_start[1]-windowsize)) & (self.position["pos"]<(pos_end[1]+windowsize))
            I =  sp.nonzero(I)[0]
            idx_start = I.min()
            idx_end = I.max()
        elif (chrom is not None):
            I = self.position["chrom"]==chrom

            idx_start = I.min()
            idx_end = I.max()
        else:
            idx_start=None
            idx_end=None
        return idx_start,idx_end

    def getCovariance(self,sample_idx=None,normalize=True,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,blocksize=None,X=None,snp_idx=None,windowsize=0,**kw_args):
        """calculate the empirical genotype covariance in a region"""
        if X is not None:
            if X.dtype!= sp.float64:
                X= sp.array(X,dtype= sp.float64)
            K=X.dot(X.T)
            Nsnp=X.shape[1]
        else:
            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) and (chrom is not None):
                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,windowsize=windowsize)

            [N,M]=self.geno_matrix.shape
            if blocksize is None:
                blocksize=M
            if idx_start is None:
                idx_start=0
            if idx_end is None:
                idx_end=M
            nread = idx_start
            K=None
            Nsnp=idx_end-idx_start
            while nread<idx_end:
                thisblock=min(blocksize,idx_end-nread)
                X=self.getGenotypes(sample_idx=sample_idx,idx_start=nread,idx_end=(nread+thisblock),center=center,unit=unit,impute_missing=True,**kw_args)
                if X.dtype!= sp.float64:
                    X= sp.array(X,dtype= sp.float64)
                if K is None:
                    K=X.dot(X.T)
                else:
                    K+=X.dot(X.T)
                nread+=thisblock
        if normalize:
            K/=(K.diagonal().mean())
        #else:#divide by number of SNPs in K
        #    K/=Nsnp
        return K

    def getGenoID(self,snp_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
        """get genotype IDs.
        Optionally the indices for loading subgroups the genotype IDs for all people
        can be given in one out of three ways:
        - 0-based indexing (idx_start-idx_end)
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)
            pos_cum_start:   cumulative position based selection (start position)
            pos_cum_end:   cumulative position based selection (end position)

        Returns:
            ID:         scipy.array of genotype IDs (e.g. rs IDs)
        """
        #position based matching?
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
        if "genotype_id" in self.geno.keys():
            if (idx_start is not None) & (idx_end is not None):
                return self.geno["genotype_id"][idx_start:idx_end]
            elif snp_idx is not None:
                return self.geno["genotype_id"][snp_idx]
            else:
                return self.geno["genotype_id"][:]
        else:
            if (idx_start is not None) & (idx_end is not None):
                return  sp.arange(idx_start,idx_start)
            elif snp_idx is not None:
                return  sp.arange(self.geno_matrix.shape[1])[snp_idx]
            else:
                return  sp.arange(self.geno_matrix.shape[1])

    def getPos(self,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,pos_cum_start=None,pos_cum_end=None):
        """
        get the positions of the genotypes

        Returns:
            chromosome
            position
            cumulative_position
        """
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
        if (idx_start is not None) & (idx_end is not None):
            return self.position.iloc[idx_start:idx_end]
        elif snp_idx is not None:
            return self.position.iloc[snp_idx]
        else:
            return self.position


    def getIcis_geno(self,geneID,cis_window=50E3):
        """ if eqtl==True it returns a bool vec for cis """
        assert self.eqtl == True, 'Only for eqtl data'
        index = self.geneID==geneID
        [_chrom,_gene_start,_gene_start] = self.gene_pos[index][0,:]
        Icis = (self.geno_chrom==_chrom)*(self.geno_pos>=_gene_start-cis_window)*(self.geno_pos<=_gene_start+cis_window)
        return Icis

class genotype_reader_tables():
    def __init__(self,file_name):
        self.file_name = file_name
        self.load()

    def load(self,cache_genotype=False,cache_phenotype=True):
        """load data file

        Args:
            cache_genotype:     load genotypes fully into memory (default: False)
            cache_phenotype:    load phentopyes fully intro memry (default: True)
        """
        import tables
        #self.f = h5py.File(self.file_name,'r')
        #self.store = pd.HDFStore(self.file_name,'r')
        self.f = tables.openFile(self.file_name,'r')
        self.geno  = self.f.root.genotype

        #parse out these we always need for convenience
        self.geno_matrix = self.geno.matrix
        #dimensions
        self.num_samples,self.num_snps = self.geno_matrix.shape

        self.sample_ID = self.geno.row_header.sample_ID[:]
        position = {
            "chrom" : self.geno.col_header.chrom[:],
            "pos"   : self.geno.col_header.pos[:],
            }

        if 'pos_cum' in self.geno.col_header:
            position["pos_cum"]   = self.geno.col_header.pos_cum[:]
        #else:
        #    position["geno_pos_cum"] = None

        if 'geno_ID' in self.geno.col_header:
            self.geno_ID   = self.geno.col_header.geno_ID[:]
        else:
            self.geno_ID =  sp.arange(self.num_snps)
        self.position = pd.DataFrame(data=position,index=self.geno_ID)
        #cache?
        if cache_genotype:
            self.geno_matrix = self.geno_matrix[:]



    def getGenotypes(self,sample_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,center=True,unit=True,pos_cum_start=None,pos_cum_end=None,impute_missing=False,snp_idx=None):
        """load genotypes.
        Optionally the indices for loading subgroups the genotypes for all people
        can be given in one out of three ways:
        - 0-based indexing (idx_start-idx_end)
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)
            pos_cum_start:   cumulative position based selection (start position)
            pos_cum_end:   cumulative position based selection (end position)
            impute_missing: Boolean indicator variable if missing values should be imputed

        Returns:
            X:          scipy.array of genotype values
        """
        #position based matching?
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) & (chrom is not None)) or ((pos_cum_start is not None) & (pos_cum_end is not None)):
            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,pos_cum_start=pos_cum_start,pos_cum_end=pose_cum1)
        #index based matching?
        if (idx_start is not None) & (idx_end is not None):
            X = self.geno_matrix[:,idx_start:idx_end]
        elif snp_idx is not None:
            X = self.geno_matrix[:,snp_idx]
        else:
            X = self.geno_matrix[:,:]
        if sample_idx is not None:
            X=X[sample_idx]
        if impute_missing:
            X = du.imputeMissing(X,center=center,unit=unit)
        return X

    def getGenoIndex(self,pos_start=None,pos_end=None,chrom=None,windowsize=0):
        """computes 0-based genotype index from position of cumulative position.
        Positions can be given in one out of two ways:
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)

        Returns:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
        """
        if (pos_start is not None) & (pos_end is not None):
            assert pos_start[0] == pos_end[0], "chromosomes don't match between start and end position"
            I = self.position["chrom"]==pos_start[0]
            I = I & (self.position["pos"]>=(pos_start[1]-windowsize)) & (self.position["pos"]<(pos_end[1]+windowsize))
            I =  sp.nonzero(I)[0]
            idx_start = I.min()
            idx_end = I.max()
        elif chrom is not None:
            I = self.position["chrom"]==chrom
            I =  sp.nonzero(I)[0]
            if I.size==0:
                return None
            idx_start = I.min()
            idx_end = I.max()
        else:
            idx_start=None
            idx_end=None
        return idx_start,idx_end

    def getCovariance(self,sample_idx=None,normalize=False,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0,center=True,unit=True,blocksize=5000,X=None,snp_idx=None,**kw_args):
        """calculate the empirical genotype covariance in a region"""
        if X is not None:
            if X.dtype!= sp.float64:
                X= sp.array(X,dtype= sp.float64)
            K=X.dot(X.T)
            Nsnp=X.shape[1]
        else:
            if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
                idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)

            [N,M]=self.geno_matrix.shape
            if blocksize is None:
                blocksize=M
            if idx_start is None:
                idx_start=0
            if idx_end is None:
                idx_end=M
            nread = idx_start
            K=None
            Nsnp=idx_end-idx_start
            while nread<idx_end:
                thisblock=min(blocksize,idx_end-nread)
                X=self.getGenotypes(sample_idx=sample_idx,idx_start=nread,idx_end=(nread+thisblock),center=center,unit=unit,impute_missing=True,**kw_args)
                if X.dtype!= sp.float64:
                    X= sp.array(X,dtype= sp.float64)
                if K is None:
                    K=X.dot(X.T)
                else:
                    K+=X.dot(X.T)
                nread+=thisblock
        if normalize:
            K/=(K.diagonal().mean())
        #else:#divide by number of SNPs in K
        #    K/=Nsnp
        return K

    def getGenoID(self,snp_idx=None,idx_start=None,idx_end=None,pos_start=None,pos_end=None,chrom=None,windowsize=0):
        """get genotype IDs.
        Optionally the indices for loading subgroups the genotype IDs for all people
        can be given in one out of three ways:
        - 0-based indexing (idx_start-idx_end)
        - position (pos_start-pos_end on chrom)
        - cumulative position (pos_cum_start-pos_cum_end)
        If all these are None (default), then all genotypes are returned

        Args:
            idx_start:         genotype index based selection (start index)
            idx_end:         genotype index based selection (end index)
            pos_start:       position based selection (start position)
            pos_end:       position based selection (end position)
            chrom:      position based selection (chromosome)

        Returns:
            ID:         scipy.array of genotype IDs (e.g. rs IDs)
        """
        #position based matching?
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)
        if "genotype_id" in self.geno:
            if (idx_start is not None) & (idx_end is not None):
                return self.geno.genotype_id[idx_start:idx_end]
            elif snp_idx is not None:
                return self.geno.genotype_id[snp_idx]
            else:
                return self.geno.genotype_id[:]
        else:
            if (idx_start is not None) & (idx_end is not None):
                return  sp.arange(idx_start,idx_start)
            elif snp_idx is not None:
                return  sp.arange(self.geno_matrix.shape[1])[snp_idx]
            else:
                return  sp.arange(self.geno_matrix.shape[1])

    def getPos(self,idx_start=None,idx_end=None,snp_idx=None,pos_start=None,pos_end=None,chrom=None,windowsize=0):
        """
        get the positions of the genotypes

        Returns:
            chromosome
            position
            cumulative_position
        """
        if (idx_start is None) and (idx_end is None) and ((pos_start is not None) & (pos_end is not None) ) or (chrom is not None):
            idx_start,idx_end=self.getGenoIndex(pos_start=pos_start,pos_end=pos_end,chrom=chrom,windowsize=windowsize)
        if (idx_start is not None) & (idx_end is not None):
            return self.position.iloc[idx_start:idx_end]
        elif snp_idx is not None:
            return self.position.iloc[snp_idx]
        else:
            return self.position


"""
    def to_table(self, filename):
        file = tables.open
        pass

class Genotype(tables.IsDescription):
    index       = tables.UInt64Col()    # running index (64-bit integer)
    id          = tables.StringCol(16)  # e.g. rs-id  (16-character String)
    chromosome  = tables.UInt8Col()     # Unsigned short integer
    bp_position = tables.UInt32Col()    # unsigned byte
    values      = tables.CArray()

"""



import numpy as np
import scipy.linalg as la
import scipy.stats as stats
from limix.core.cobj import *
from limix.core.mean.mean import compute_X1KX2, compute_XYA
import limix.utils.psd_solve as psd_solve

class LmmKronecker(cObject):
    pass

    def __init__(self, gp=None):
        '''
        Input:
        forcefullrank   : if True, then the code always computes K and runs cubically
					        (False)
        '''
        self._gp = gp
        
        gp.set_reml(False)#currently only ML is supported
        self._LL_0 = self._gp.LML()
        #self.clear_cache('LL_snps','test_snps')
    
    def addFixedEffect(self,F,A=None):

        self._gp.mean.addFixedEffect(F=F, A=A)
        self._LL_0 = self._gp.LML()
        #self.clear_cache('LL_snps','test_snps')
        pass

    def LL_snps(self, snps, Asnps=None, inter=None, identity_trick=True):
        """
        compute log likelihood for SNPs
        """

        #TODO: block diagonal Areml computations efficient
        if Asnps is not None:
            if (Asnps.shape[0]==Asnps.shape[1]) and Asnps == np.eye(Asnps.shape[0]):
                pass
        LL_snps = np.zeros(snps.shape[1])
        LL_new = np.zeros(snps.shape[1])
        LL_snps_0 = np.zeros(snps.shape[1])
        identity_trick_prev = self._gp.mean.identity_trick
        self._gp.mean.use_identity_trick(identity_trick=identity_trick)
        if not identity_trick:
            if Asnps is not None:
                Asnps.dot(self._gp.mean.Lc.T)
            else:
                Asnps = self._gp.mean.Lc.T
        if inter is None:
            LL_snps_0[:] = self._LL_0
            for i_snp in xrange(snps.shape[1]):

                LL_snps[i_snp],beta = self.LML_blockwise(snp=np.dot(self._gp.mean.Lr,snps[:,i_snp:i_snp+1]), Asnp=Asnps)
                if False and (Asnps is None) and (self._gp.mean):
                    LL_ = self.LML_snps_blockwise_any_singlesnp(snp=np.dot(self._gp.mean.Lr,snps[:,i_snp:i_snp+1]))

        else:
            for i_snp in xrange(snps.shape[1]):
                self._gp.mean.addFixedEffect(F=snps[:,i_snp:i_snp+1], A=Asnps)
                LL_snps_0[i_snp] = self._gp.LML()
                self._gp.mean.addFixedEffect(F=snps[:,i_snp:i_snp+1]*inter, A=Asnps)
                LL_snps[i_snp] = self._gp.LML()
                self._gp.mean.removeFixedEffect()
                self._gp.mean.removeFixedEffect()
        self._gp.mean.use_identity_trick(identity_trick=identity_trick_prev)
        return LL_snps,LL_snps_0


    def LML_snps_blockwise_any_multisnp(self, snps ):
        """
        calculate LML
        The beta of the SNP tested is computed using blockwise matrix inversion.
        """
        self._gp._update_cache()
        
        #1. const term
        lml  = self._gp.N*self._gp.P*np.log(2.0*np.pi)

        #2. logdet term
        lml += np.sum(np.log(self._gp.cache['Sc2']))*self._gp.N + np.log(self._gp.cache['s']).sum()

        #3. quadratic term
        XKY = self._gp.mean.compute_XKY(M=self._gp.mean.Yhat())
        beta = self._gp.mean.Areml_solve(XKY)
        var_total = (self._gp.mean.Yhat()*self._gp.mean.Ystar()).sum()
        var_expl_cov = (XKY*beta).sum()
        
        
        #use blockwise matrix inversion
        #[  Areml,          XcovarXsnp
        #   XcovarXsnp.T    XsnpXsnp    ]
        var_expl_snp = np.zeros((snps.shape[1]))
        var_expl_up = 0.0

        #trivially parallelizable:
        for p in xrange(self._gp.P):
            Dsnps = snps * D[:,c:c+1]
            XsnpKXsnp = (Dsnps * snps).sum(0)
            XcovarXsnp = np.zeros((self._gp.mean.dof,snps.shape[1]))
            start = 0
            if 0:
                for term in xrange(self._gp.mean.len):
                    n_effs_term = self._gp.mean.Fstar()[term].shape[1]
            
                    Astar_term = self._gp.mean.Astar()[term]
                    n_effs_term *= Astar_term.shape[0]
                    stop = start + n_effs_term
                    block = self._gp.mean.Fstar()[term].T.dot(Dsnps)
                    block = np.kron(A2[:,c:c+1],block)
                    XcovarXsnp[start:stop,:] = block
                    start=stop
            XanyXsnp = self._gp.mean.Fstar_any.T.dot(Dsnps)

            #For any effect covariates perform low rank update for each phenotype
            
            #compute DiC:
            DiC = self._gp.mean.Areml_solver.solve(b_any=XanyXsnp,p=p)

            #compute the Schur complement:
            up_schur = (XanyXsnp * DiC).sum(0)
            schur = XsnpKXsnp - up_schur
            
            beta_snp = (XsnpKY - DiC.T.dot(XKY)) / schur
            beta_up = DiC.T.dot(beta_snp)

            var_expl_snp += (XsnpKY * beta_snp)
            var_expl_up += (XKY * beta_up).sum()

        var_res = var_total - var_expl_snp - var_expl_cov - var_expl_up
        
        lml += var_res
        lml *= 0.5
            
        return lml,beta_all

    def LML_snps_blockwise_any_singlesnp(self, snp ):
        """
        calculate LML
        The beta of the SNP tested is computed using blockwise matrix inversion.
        """
        self._gp._update_cache()
        
        #1. const term
        lml  = self._gp.N*self._gp.P*np.log(2.0*np.pi)

        #2. logdet term
        lml += np.sum(np.log(self._gp.cache['Sc2']))*self._gp.N + np.log(self._gp.cache['s']).sum()

        #3. quadratic term
        XKY = self._gp.mean.compute_XKY(M=self._gp.mean.Yhat())
        beta = self._gp.mean.Areml_solve(XKY)
        var_total = (self._gp.mean.Yhat()*self._gp.mean.Ystar()).sum()
        var_expl_cov = (XKY*beta).sum()
        
        
        #use blockwise matrix inversion
        #[  Areml,          XcovarXsnp
        #   XcovarXsnp.T    XsnpXsnp    ]
        Dsnp = snp * self._gp.mean.D
        XsnpKXsnp = (Dsnp * snp).sum(0)
        XcovarXsnp = np.zeros((self._gp.mean.dof,snps.shape[1]))
        start = 0
        if 0:
            for term in xrange(self._gp.mean.len):
                n_effs_term = self._gp.mean.Fstar()[term].shape[1]
            
                Astar_term = self._gp.mean.Astar()[term]
                n_effs_term *= Astar_term.shape[0]
                stop = start + n_effs_term
                block = self._gp.mean.Fstar()[term].T.dot(Dsnps)
                block = np.kron(A2[:,c:c+1],block)
                XcovarXsnp[start:stop,:] = block
                start=stop
        XanyXsnp = self._gp.mean.Fstar_any.T.dot(Dsnps)
        XKXsnp = XXX
        #For any effect covariates perform low rank update for each phenotype
            
        #compute DiC:
        DiC = self._gp.mean.Areml_solver.solve(b_any=XanyXsnp)

        #compute the Schur complement:
        up_schur = XanyXsnp.T.dot(DiC)
        schur = XsnpKXsnp - up_schur
            
        beta_snp = (XsnpKY - DiC.T.dot(XKY)) / schur
        beta_up = DiC.T.dot(beta_snp)

        var_expl_snp = (XsnpKY * beta_snp).sum()
        var_expl_up = (XKY * beta_up).sum()

        var_res = var_total - var_expl_snp - var_expl_cov - var_expl_up
        
        lml += var_res
        lml *= 0.5
            
        return lml,beta_all


    def LML_blockwise(self, snp, Asnp=None, *kw_args):
        """
        calculate LML
        The beta of the SNP tested is computed using blockwise matrix inversion.
        """
        self._gp._update_cache()
        
        if Asnp is None:
            nW_Asnp = self._gp.mean.Ystar().shape[1]	
        else:
            nW_Asnp = Asnp.shape[1]
            	
        #1. const term
        lml  = self._gp.N*self._gp.P*np.log(2.0*np.pi)

        #2. logdet term
        lml += np.sum(np.log(self._gp.cache['Sc2']))*self._gp.N + np.log(self._gp.cache['s']).sum()

        #3. quadratic term
        #quad1 = (self._gp.mean.Zstar(identity_trick=identity_trick)*self._gp.mean.DLZ(identity_trick=identity_trick)).sum()
        
        XKY = self._gp.mean.compute_XKY(M=self._gp.mean.Yhat())
        import ipdb; ipdb.set_trace()
        beta = self._gp.mean.Areml_solve(XKY)
        var_total = (self._gp.mean.Yhat()*self._gp.mean.Ystar()).sum()
        var_expl = (XKY*beta).sum()
        
        
        #use blockwise matrix inversion
        #[  Areml,          XcovarXsnp
        #   XcovarXsnp.T    XsnpXsnp    ]
        
        XsnpXsnp = compute_X1KX2(Y=self._gp.mean.Ystar(), D=self._gp.mean.D, X1=snp, X2=snp, A1=Asnp, A2=Asnp)
        XcovarXsnp = np.zeros((self._gp.mean.n_fixed_effs,nW_Asnp*snp.shape[1]))
        start = 0
        for term in xrange(self._gp.mean.n_terms):
            n_effs_term = self._gp.mean.Fstar()[term].shape[1]
            
            if self._gp.mean.identity_trick and self._gp.mean.A_identity[term]:
                Astar_term = None
                n_effs_term *= self._gp.P
            else:
                Astar_term = self._gp.mean.Astar()[term]
                n_effs_term *= Astar_term.shape[0]
            stop = start + n_effs_term
            block = compute_X1KX2(Y=self._gp.mean.Ystar(), D=self._gp.mean.D, X1=self._gp.mean.Fstar()[term], X2=snp, A1=Astar_term, A2=Asnp)
            XcovarXsnp[start:stop,:] = block
            start=stop
        AXcovarXsnp = self._gp.mean.Areml_solve(XcovarXsnp)
        XsnpXsnp_ = XsnpXsnp - XcovarXsnp.T.dot(AXcovarXsnp)
        
        #compute a
        snpKY = compute_XYA(DY=self._gp.mean.Yhat(), X=snp, A=Asnp).ravel(order='F')
        

        XsnpXsnp_solver = psd_solve.psd_solver(XsnpXsnp_, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False)
        #solve XsnpXsnp \ AXcovarXsnp*beta

        if 0:
            beta_snp1 = XsnpXsnp_solver.solve(snpKY,overwrite_b=False)
            beta_snp2 = XsnpXsnp_solver.solve(AXcovarXsnp.T.dot(XKY),overwrite_b=False)
            beta_snp = beta_snp1 - beta_snp2
        beta_snp = XsnpXsnp_solver.solve(snpKY - AXcovarXsnp.T.dot(XKY),overwrite_b=False)

        #solve XsnpXsnp \ a
        if 0:
            beta_up1 = AXcovarXsnp.dot(beta_snp2) #AXcovarXsnp.dot(beta_snp)#self._gp.mean.Areml_solve(XcovarXsnp.dot(-beta_snp),identity_trick=identity_trick)#This is not correct
            beta_up2 = AXcovarXsnp.dot(beta_snp1)
            beta_up = beta_up1 - beta_up2
            beta_new = beta+beta_up
        beta_up = - AXcovarXsnp.dot(beta_snp)
        var_expl_update = (XKY*beta_up).sum()
        var_expl_snp = (snpKY*beta_snp).sum()

        var_expl_all = var_expl + var_expl_snp+var_expl_update
        beta_all = np.concatenate([beta+beta_up,beta_snp])
        var_res = var_total - var_expl_snp - var_expl_update - var_expl

        lml += var_res
        lml *= 0.5
        #if (var_expl_all)<var_expl-1e-4:
        if 0: #debugging
            Areml1 = np.concatenate((self._gp.mean.Areml(),XcovarXsnp),1)
            Areml2 = np.concatenate((XcovarXsnp.T, XsnpXsnp),1)
            Areml_all = np.concatenate((Areml1,Areml2),0)
            XKY_all = np.concatenate((XKY,snpKY),0)

            Areml_all_solver = psd_solve.psd_solver(Areml_all, lower=True, threshold=1e-10,check_finite=True,overwrite_a=False)
            beta_all_ = Areml_all_solver.solve(XKY_all)
            Areml_all_inv = Areml_all_solver.solve(np.eye(XKY_all.shape[0]))
            Areml_inv = self._gp.mean.Areml_inv()
            Aremlinv_up = AXcovarXsnp.dot(XsnpXsnp_solver.solve(AXcovarXsnp.T))
            Areml_inv_part_1 = Areml_inv + Aremlinv_up
            Areml_inv_part_2 = XsnpXsnp_solver.solve(np.eye(XsnpXsnp.shape[0]))
            Areml_inv_part_3 = - XsnpXsnp_solver.solve(AXcovarXsnp.T)
            Areml_all_inv_1 = np.concatenate((Areml_inv_part_1,Areml_inv_part_3.T),1)
            Areml_all_inv_2 = np.concatenate((Areml_inv_part_3,Areml_inv_part_2),1)
            Areml_all_inv_ = np.concatenate((Areml_all_inv_1,Areml_all_inv_2),0)
            beta_all__ = np.dot(Areml_all_inv_,XKY_all)
            diff_areml =  np.absolute(Areml_all_inv_-Areml_all_inv).sum()
            diff_beta_ = np.absolute(beta_all_-beta_all).sum()
            diff_beta__ = np.absolute(beta_all__-beta_all).sum()
            diff_beta_hat = np.absolute(beta-beta___).sum()
            print "var_expl = %.4f" % var_expl
            print "var_expl_update = %.4f" % var_expl_update
            print "var_expl_snp = %.4f",  var_expl_snp
            print "absdiff Areml = %.5f",diff_areml
            print "absdiff beta_ = %.5f",diff_beta_
            print "absdiff beta__ = %.5f",diff_beta__
            print "absdiff beta_hat = %.5f",diff_beta_hat
            import ipdb;ipdb.set_trace()
            
        return lml,beta_all

    def test_snps(self, snps, Asnps=None, inter=None, identity_trick=False):
        """
        test snps for association
        """
        if Asnps is None:
            dof = self._gp.mean.P
        else:
            dof = Asnps.shape[0]
        LL_snps, LL_snps_0 = self.LL_snps(snps=snps,Asnps=Asnps, inter=inter, identity_trick=identity_trick)
        LRT = 2.0 * (LL_snps_0 - LL_snps)
        pv = stats.chi2.sf(LRT,dof)
        return pv,LL_snps,LL_snps_0






"""Variance Decomposition testing code"""
import unittest
import scipy as SP
import numpy as np
import scipy.stats
import pdb
import os
import sys
import limix.deprecated as dlimix
import limix.deprecated.modules.lmmlasso as lmmlasso
from limix.test import data


class Lmmlasso_test(unittest.TestCase):
    """test class for lmm-lasso
    """

    def genGeno(self):
        X  = (SP.rand(self.N,self.S)<0.2)*1.
        X -= X.mean(0)
        X /= X.std(0)
        self.D['X'] = X

    def genKernel(self):
        X  = (SP.rand(self.N,10)<0.2)*1.
        K  = SP.dot(X,X.T)
        K /= SP.diag(K).mean()
        K += 1e-3*SP.eye(self.N)
        self.D['K'] = K


    def genPheno(self):

        idx_causal = SP.random.randint(0,self.S,10)
        sigma_g = 0.25
        sigma_e = 0.25
        sigma_f = 0.50

        u = SP.random.multivariate_normal(SP.zeros(self.N),self.D['K'])
        u*= SP.sqrt(sigma_g)/u.std()

        e = SP.random.randn(self.N)
        e*= SP.sqrt(sigma_e)/e.std()

        f = SP.sum(self.D['X'][:,idx_causal],axis=1)
        f*= SP.sqrt(sigma_f)/f.std()

        y = u + e + f
        self.D['y']= y
        self.D['causal_idx'] = idx_causal




    def setUp(self):
        #check: do we have a csv File?
        self.dir_name = os.path.dirname(os.path.realpath(__file__))
        self.dataset = os.path.join(self.dir_name,'lmmlasso')

        if (not os.path.exists(self.dataset)) or 'recalc' in sys.argv:
            if not os.path.exists(self.dataset):
                os.makedirs(self.dataset)
            SP.random.seed(1)
            self.N = 500
            self.S = 100
            self.D = {}
            self.genGeno()
            self.genKernel()
            self.genPheno()
            self.generate = True
        else:
            self.generate=False
            self.D = data.load(self.dataset)
            self.N = self.D['X'].shape[0]
            self.S = self.D['X'].shape[1]

        self.lmmlasso = lmmlasso.LmmLasso()


    def test_fit(self):
        """ test fitting """
        self.lmmlasso.set_params(alpha=1e-1)
        self.lmmlasso.fit(self.D['X'],self.D['y'],self.D['K'])
        params = self.lmmlasso.coef_
        yhat = self.lmmlasso.predict(self.D['X'],self.D['K'])

        if self.generate:
            self.D['params_true'] = params
            self.D['yhat'] = yhat
            data.dump(self.D,self.dataset)
            self.generate=False
        params_true = self.D['params_true']
        yhat_true   = self.D['yhat']

        RV = ((SP.absolute(params)-SP.absolute(params_true))**2).max()
        np.testing.assert_almost_equal(RV, 0., decimal=4)

        RV = ((SP.absolute(yhat)-SP.absolute(yhat_true))**2).max()
        np.testing.assert_almost_equal(RV, 0., decimal=2)



if __name__ == '__main__':
    unittest.main()



import os
import pdb
import sys
import csv
import numpy as np
from optparse import OptionParser
import time
import limix
from read_utils import readNullModelFile
from read_utils import readWindowsFile
from read_utils import readCovarianceMatrixFile
from read_utils import readCovariatesFile
from read_utils import readPhenoFile
import plink_reader
import scipy as sp
import warnings

def scan(bfile,Y,cov,null,wnds,minSnps,i0,i1,perm_i,resfile,F,colCovarType_r='lowrank',rank_r=1):

    if perm_i is not None:
        print 'Generating permutation (permutation %d)'%perm_i
        np.random.seed(perm_i)
        perm = np.random.permutation(Y.shape[0])

    mtSet = limix.MTSet(Y=Y, S_R=cov['eval'], U_R=cov['evec'], F=F, rank=rank_r)
    mtSet.setNull(null)
    bim = plink_reader.readBIM(bfile,usecols=(0,1,2,3))
    fam = plink_reader.readFAM(bfile,usecols=(0,1))
   
    print 'fitting model'
    wnd_file = csv.writer(open(resfile,'wb'),delimiter='\t')
    for wnd_i in range(i0,i1):
        print '.. window %d - (%d, %d-%d) - %d snps'%(wnd_i,int(wnds[wnd_i,1]),int(wnds[wnd_i,2]),int(wnds[wnd_i,3]),int(wnds[wnd_i,-1]))
        if int(wnds[wnd_i,-1])<minSnps:
            print 'SKIPPED: number of snps lower than minSnps'
            continue
        #RV = bed.read(PositionRange(int(wnds[wnd_i,-2]),int(wnds[wnd_i,-1])))
        RV = plink_reader.readBED(bfile, useMAFencoding=True, blocksize = 1, start = int(wnds[wnd_i,4]), nSNPs = int(wnds[wnd_i,5]), order  = 'F',standardizeSNPs=False,ipos = 2,bim=bim,fam=fam)
        
        Xr = RV['snps']
        if perm_i is not None:
            Xr = Xr[perm,:]
        rv = mtSet.optimize(Xr)
        line = np.concatenate([wnds[wnd_i,:],rv['LLR']])
        wnd_file.writerow(line)
    pass

def analyze(options):

    # load data
    print 'import data'
    if options.cfile is None:
        cov = {'eval':None,'evec':None}
        warnings.warn('warning: cfile not specifed, a one variance compoenent model will be considered')
    else:
        cov = readCovarianceMatrixFile(options.cfile,readCov=False)
    Y = readPhenoFile(options.pfile,idx=options.trait_idx)
    null = readNullModelFile(options.nfile)
    wnds = readWindowsFile(options.wfile)

    F = None
    if options.ffile:
        F = readCovariatesFile(options.ffile)
        #null['params_mean'] = sp.loadtxt(options.nfile + '.f0')
        

    if F is not None: assert Y.shape[0]==F.shape[0], 'dimensions mismatch'

            
    if options.i0 is None: options.i0 = 1
    if options.i1 is None: options.i1 = wnds.shape[0]

    # name of output file
    if options.perm_i is not None:
        res_dir = os.path.join(options.resdir,'perm%d'%options.perm_i)
    else:
        res_dir = os.path.join(options.resdir,'test')
    if not os.path.exists(res_dir):
        os.makedirs(res_dir)
    n_digits = len(str(wnds.shape[0]))
    fname = str(options.i0).zfill(n_digits)
    fname+= '_'+str(options.i1).zfill(n_digits)+'.res'
    resfile = os.path.join(res_dir,fname)

    # analysis
    t0 = time.time()
    scan(options.bfile,Y,cov,null,wnds,options.minSnps,options.i0,options.i1,options.perm_i,resfile,F,options.colCovarType_r,options.rank_r)
    t1 = time.time()
    print '... finished in %s seconds'%(t1-t0)




"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import limix.deprecated as dlimix
from covar import Acovar_test

class CCovSqexpARD_test(unittest.TestCase,Acovar_test):
    """test class for CCovSqexpARD"""
    def setUp(self):
        SP.random.seed(1)
        self.n=10
        self.n_dim=10
        X=SP.rand(self.n,self.n_dim)
        self.C = dlimix.CCovSqexpARD(self.n_dim)
        self.name = 'CCovSqexpARD'
        self.C.setX(X)
        K = self.C.K()
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
"""
qtl.py contains wrappers around C++ Limix objects to streamline common tasks in GWAS.
"""

import numpy as np
import scipy.stats as st
import limix
import scipy as sp
import limix.deprecated
import limix.deprecated as dlimix
import limix.deprecated.utils.preprocess as preprocess
import limix.deprecated.stats.fdr as FDR
import varianceDecomposition as VAR
import time
#import qtl module for self referencing
import qtl


class lmm:
	def __init__(self, snps, pheno, K=None, covs=None, test='lrt', NumIntervalsDelta0=100, NumIntervalsDeltaAlt=100, searchDelta=False, verbose=None):
		"""
		Univariate fixed effects linear mixed model test for all SNPs

		If phenotypes have missing values, then the subset of individuals used for each phenotype column
		will be subsetted

		Args:
			snps:   [N x S] np.array of S SNPs for N individuals
			pheno:  [N x P] np.array of P phenotype sfor N individuals
			K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
							If not provided, then linear regression analysis is performed
			covs:   [N x D] np.array of D covariates for N individuals
			test:   'lrt' for likelihood ratio test (default) or 'f' for F-test
			NumIntervalsDelta0:     number of steps for delta optimization on the null model (100)
			NumIntervalsDeltaAlt:   number of steps for delta optimization on the alt. model (100), requires searchDelta=True to have an effect.
			searchDelta:     Carry out delta optimization on the alternative model? if yes We use NumIntervalsDeltaAlt steps
			verbose: print verbose output? (False)
		"""
		#create column of 1 for fixed if nothing provide
		if len(pheno.shape)==1:
			pheno = pheno[:,sp.newaxis]

		self.verbose = dlimix.getVerbose(verbose)
		self.snps = snps
		self.pheno = pheno
		self.K = K
		self.covs = covs
		self.test = test
		self.NumIntervalsDelta0 = NumIntervalsDelta0
		self.NumIntervalsDeltaAlt = NumIntervalsDeltaAlt
		self.searchDelta = searchDelta
		self.verbose = verbose
		self.N       = self.pheno.shape[0]
		self.P       = self.pheno.shape[1]
		self.Iok     = ~(np.isnan(self.pheno).any(axis=1))
		if self.K is None:
			self.searchDelta=False
			self.K = np.eye(self.snps.shape[0])
		if self.covs is None:
			self.covs = np.ones((self.snps.shape[0],1))

		self._lmm = None
		#run
		self.verbose = verbose
		self.process()

	def process(self):
		t0 = time.time()
		if self._lmm is None:
			self._lmm = limix.deprecated.CLMM()
			self._lmm.setK(self.K)
			self._lmm.setSNPs(self.snps)
			self._lmm.setPheno(self.pheno)
			self._lmm.setCovs(self.covs)
			if self.test=='lrt':
				self._lmm.setTestStatistics(self._lmm.TEST_LRT)
			elif self.test=='f':
				self._lmm.setTestStatistics(self._lmm.TEST_F)
			else:
				print self.test
				raise NotImplementedError("only f and lrt are implemented")
			#set number of delta grid optimizations?
			self._lmm.setNumIntervals0(self.NumIntervalsDelta0)
			if self.searchDelta:
				self._lmm.setNumIntervalsAlt(self.NumIntervalsDeltaAlt)
			else:
				self._lmm.setNumIntervalsAlt(0)

		if not np.isnan(self.pheno).any():
			#process
			self._lmm.process()
			self.pvalues = self._lmm.getPv()
			self.beta_snp = self._lmm.getBetaSNP()
			self.beta_ste = self._lmm.getBetaSNPste()
			self.ldelta_0 = self._lmm.getLdelta0()
			self.ldelta_alt = self._lmm.getLdeltaAlt()
			self.NLL_0 = self._lmm.getNLL0()
			self.NLL_alt = self._lmm.getNLLAlt()
		else:
			if self._lmm is not None:
				raise Exception('cannot reuse a CLMM object if missing variables are present')
			else:
				self._lmm = limix.deprecated.CLMM()
			#test all phenotypes separately
			self.pvalues = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.beta_snp = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.beta_ste = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.ldelta_0 = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.ldelta_alt = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.NLL_0 = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.NLL_alt = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			self.test_statistics = np.zeros((self.phenos.shape[1],self.snps.shape[1]))
			for ip in np.arange(self.phenos.shape[1]):
				pheno_ = self.phenos[:,ip]
				i_nonz = ~(pheno_.isnan())

				self._lmm.setK(self.K[i_nonz,i_nonz])
				self._lmm.setSNPs(self.snps[i_nonz])
				self._lmm.setPheno(pheno_[i_nonz,np.newaxis])
				self._lmm.setCovs(self.covs[i_nonz])
				self._lmm.process()
				self.pvalues[ip:ip+1] = self._lmm.getPv()
				self.beta_snp[ip:ip+1] = self._lmm.getBetaSNP()
				self.beta_ste[ip:ip+1] = self._lmm.getBetaSNPste()
				self.ldelta_0[ip:ip+1] = self._lmm.getLdelta0()
				self.ldelta_alt[ip:ip+1] = self._lmm.getLdeltaAlt()
				self.NLL_0[ip:ip+1] = self._lmm.getNLL0()
				self.NLL_alt[ip:ip+1] = self._lmm.getNLLAlt()
				self.test_statistics[ip:ip+1] = self._lmm.getTestStatistics()
				pass
		if self._lmm.getTestStatistics() == self._lmm.TEST_LRT and self.test != "lrt":
			raise NotImplementedError("only f and lrt are implemented")
		elif self._lmm.getTestStatistics() == self._lmm.TEST_F and self.test != "f":
			raise NotImplementedError("only f and lrt are implemented")

		if self._lmm.getTestStatistics() == self._lmm.TEST_F:
			self.test_statistics = (self.beta_snp*self.beta_snp)/(self.beta_ste*self.beta_ste)
		if self._lmm.getTestStatistics() == self._lmm.TEST_LRT:
			self.test_statistics = 2.0 * (self.NLL_0 - self.NLL_alt)
		t1=time.time()

		if self.verbose:
			print ("finished GWAS testing in %.2f seconds" %(t1-t0))

	def setCovs(self,covs):
		self._lmm.setCovs(covs)

	def getBetaSNP(self):
		return self.beta_snp

	def getPv(self):
		"""
		Returns:
			[P x S] np.array of P-values
		"""
		return self.pvalues

def test_lm(snps,pheno, covs=None, test='lrt',verbose=None):
	"""
	Univariate fixed effects linear model test for all SNPs
	(wrapper around test_lmm, using identity kinship)

	If phenotypes have missing values, then the subset of individuals used for each phenotype column
	will be subsetted

	Args:
		snps:   [N x S] np.array of S SNPs for N individuals
		pheno:  [N x 1] np.array of 1 phenotype for N individuals
		covs:   [N x D] np.array of D covariates for N individuals
		test:   'lrt' for likelihood ratio test (default) or 'f' for F-test
		verbose: print verbose output? (False)

	Returns:
		limix LMM object
	"""
	lm = test_lmm(snps=snps,pheno=pheno,K=None,covs=covs, test=test,verbose=verbose)
	return lm

def test_lmm(snps,pheno,K=None,covs=None, test='lrt',NumIntervalsDelta0=100,NumIntervalsDeltaAlt=100,searchDelta=False,verbose=None):
	"""
	Univariate fixed effects linear mixed model test for all SNPs

	If phenotypes have missing values, then the subset of individuals used for each phenotype column
	will be subsetted

	Args:
		snps:   [N x S] np.array of S SNPs for N individuals
		pheno:  [N x 1] np.array of 1 phenotype for N individuals
		K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
						If not provided, then linear regression analysis is performed
		covs:   [N x D] np.array of D covariates for N individuals
		test:   'lrt' for likelihood ratio test (default) or 'f' for F-test
		NumIntervalsDelta0:     number of steps for delta optimization on the null model (100)
		NumIntervalsDeltaAlt:   number of steps for delta optimization on the alt. model (100), requires searchDelta=True to have an effect.
		searchDelta:     Carry out delta optimization on the alternative model? if yes We use NumIntervalsDeltaAlt steps
		verbose: print verbose output? (False)

	Returns:
		LMM object
	"""
	lmm = qtl.lmm(snps=snps, pheno=pheno, K=K, covs=covs, test=test, NumIntervalsDelta0=NumIntervalsDelta0, NumIntervalsDeltaAlt=NumIntervalsDeltaAlt, searchDelta=searchDelta, verbose=verbose)
	return lmm


def test_lmm_kronecker(snps,phenos,covs=None,Acovs=None,Asnps=None,K1r=None,K1c=None,K2r=None,K2c=None,trait_covar_type='lowrank_diag',rank=1,NumIntervalsDelta0=100,NumIntervalsDeltaAlt=100,searchDelta=False):
    """
    simple wrapper for kroneckerLMM code

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        phenos: [N x P] np.array of P phenotypes for N individuals
        covs:           list of np.arrays holding covariates. Each covs[i] has one corresponding Acovs[i]
        Acovs:          list of np.arrays holding the phenotype design matrices for covariates.
                        Each covs[i] has one corresponding Acovs[i].
        Asnps:          single np.array of I0 interaction variables to be included in the
                        background model when testing for interaction with Inters
                        If not provided, the alternative model will be the independent model
        K1r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K1c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        trait_covar_type:     type of covaraince to use. Default 'freeform'. possible values are
                        'freeform': free form optimization,
                        'fixed': use a fixed matrix specified in covar_K0,
                        'diag': optimize a diagonal matrix,
                        'lowrank': optimize a low rank matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_id': optimize a low rank matrix plus the weight of a constant diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_diag': optimize a low rank matrix plus a free diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'block': optimize the weight of a constant P x P block matrix of ones,
                        'block_id': optimize the weight of a constant P x P block matrix of ones plus the weight of a constant diagonal matrix,
                        'block_diag': optimize the weight of a constant P x P block matrix of ones plus a free diagonal matrix,
        rank:           rank of a possible lowrank component (default 1)
        NumIntervalsDelta0:  number of steps for delta optimization on the null model (100)
        NumIntervalsDeltaAlt:number of steps for delta optimization on the alt. model (100), requires searchDelta=True to have an effect.
        searchDelta:    Boolean indicator if delta is optimized during SNP testing (default False)

    Returns:
        CKroneckerLMM object
        P-values for all SNPs from liklelihood ratio test
    """
    #0. checks
    N  = phenos.shape[0]
    P  = phenos.shape[1]

    if K1r==None:
        K1r = np.dot(snps,snps.T)
    else:
        assert K1r.shape[0]==N, 'K1r: dimensions dismatch'
        assert K1r.shape[1]==N, 'K1r: dimensions dismatch'

    if K2r==None:
        K2r = np.eye(N)
    else:
        assert K2r.shape[0]==N, 'K2r: dimensions dismatch'
        assert K2r.shape[1]==N, 'K2r: dimensions dismatch'

    covs, Acovs = _updateKronCovs(covs,Acovs,N,P)

    #Asnps can be several designs
    if Asnps is None:
        Asnps = [np.ones([1,P])]
    if (type(Asnps)!=list):
        Asnps = [Asnps]
    assert len(Asnps)>0, "need at least one Snp design matrix"

    #one row per column design matrix
    pv = np.zeros((len(Asnps),snps.shape[1]))

    #1. run GP model to infer suitable covariance structure
    if K1c==None or K2c==None:
        vc = _estimateKronCovariances(phenos=phenos, K1r=K1r, K2r=K2r, K1c=K1c, K2c=K2c, covs=covs, Acovs=Acovs, trait_covar_type=trait_covar_type, rank=rank)
        K1c = vc.getTraitCovar(0)
        K2c = vc.getTraitCovar(1)
    else:
        assert K1c.shape[0]==P, 'K1c: dimensions dismatch'
        assert K1c.shape[1]==P, 'K1c: dimensions dismatch'
        assert K2c.shape[0]==P, 'K2c: dimensions dismatch'
        assert K2c.shape[1]==P, 'K2c: dimensions dismatch'

    #2. run kroneckerLMM

    lmm = limix.deprecated.CKroneckerLMM()
    lmm.setK1r(K1r)
    lmm.setK1c(K1c)
    lmm.setK2r(K2r)
    lmm.setK2c(K2c)
    lmm.setSNPs(snps)
    #add covariates
    for ic  in xrange(len(Acovs)):
        lmm.addCovariates(covs[ic],Acovs[ic])
    lmm.setPheno(phenos)


    #delta serch on alt. model?
    if searchDelta:
        lmm.setNumIntervalsAlt(NumIntervalsDeltaAlt)
    else:
        lmm.setNumIntervalsAlt(0)
    lmm.setNumIntervals0(NumIntervalsDelta0)

    for iA in xrange(len(Asnps)):
        #add SNP design
        lmm.setSNPcoldesign(Asnps[iA])
        lmm.process()
        pv[iA,:] = lmm.getPv()[0]
    return lmm,pv


def test_interaction_lmm_kronecker(snps,phenos,covs=None,Acovs=None,Asnps1=None,Asnps0=None,K1r=None,K1c=None,K2r=None,K2c=None,trait_covar_type='lowrank_diag',rank=1,NumIntervalsDelta0=100,NumIntervalsDeltaAlt=100,searchDelta=False,return_lmm=False):
    """
    I-variate fixed effects interaction test for phenotype specific SNP effects

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        phenos: [N x P] np.array of P phenotypes for N individuals
        covs:           list of np.arrays holding covariates. Each covs[i] has one corresponding Acovs[i]
        Acovs:          list of np.arrays holding the phenotype design matrices for covariates.
                        Each covs[i] has one corresponding Acovs[i].
        Asnps1:         list of np.arrays of I interaction variables to be tested for N
                        individuals. Note that it is assumed that Asnps0 is already included.
                        If not provided, the alternative model will be the independent model
        Asnps0:         single np.array of I0 interaction variables to be included in the
                        background model when testing for interaction with Inters
        K1r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K1c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        trait_covar_type:     type of covaraince to use. Default 'freeform'. possible values are
                        'freeform': free form optimization,
                        'fixed': use a fixed matrix specified in covar_K0,
                        'diag': optimize a diagonal matrix,
                        'lowrank': optimize a low rank matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_id': optimize a low rank matrix plus the weight of a constant diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_diag': optimize a low rank matrix plus a free diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'block': optimize the weight of a constant P x P block matrix of ones,
                        'block_id': optimize the weight of a constant P x P block matrix of ones plus the weight of a constant diagonal matrix,
                        'block_diag': optimize the weight of a constant P x P block matrix of ones plus a free diagonal matrix,
        rank:           rank of a possible lowrank component (default 1)
        NumIntervalsDelta0:  number of steps for delta optimization on the null model (100)
        NumIntervalsDeltaAlt:number of steps for delta optimization on the alt. model (100), requires searchDelta=True to have an effect.
        searchDelta:     Carry out delta optimization on the alternative model? if yes We use NumIntervalsDeltaAlt steps
    Returns:
        pv:     P-values of the interaction test
        pv0:    P-values of the null model
        pvAlt:  P-values of the alternative model
    """
    S=snps.shape[1]
    #0. checks
    N  = phenos.shape[0]
    P  = phenos.shape[1]

    if K1r==None:
        K1r = np.dot(snps,snps.T)
    else:
        assert K1r.shape[0]==N, 'K1r: dimensions dismatch'
        assert K1r.shape[1]==N, 'K1r: dimensions dismatch'

    if K2r==None:
        K2r = np.eye(N)
    else:
        assert K2r.shape[0]==N, 'K2r: dimensions dismatch'
        assert K2r.shape[1]==N, 'K2r: dimensions dismatch'

    covs,Acovs = _updateKronCovs(covs,Acovs,N,P)

    #Asnps can be several designs
    if (Asnps0 is None):
        Asnps0 = [np.ones([1,P])]
    if Asnps1 is None:
        Asnps1 = [np.eye([P])]
    if (type(Asnps0)!=list):
        Asnps0 = [Asnps0]
    if (type(Asnps1)!=list):
        Asnps1 = [Asnps1]
    assert (len(Asnps0)==1) and (len(Asnps1)>0), "need at least one Snp design matrix for null and alt model"

    #one row per column design matrix
    pv = np.zeros((len(Asnps1),snps.shape[1]))
    lrt = np.zeros((len(Asnps1),snps.shape[1]))
    pvAlt = np.zeros((len(Asnps1),snps.shape[1]))
    lrtAlt = np.zeros((len(Asnps1),snps.shape[1]))

    #1. run GP model to infer suitable covariance structure
    if K1c==None or K2c==None:
        vc = _estimateKronCovariances(phenos=phenos, K1r=K1r, K2r=K2r, K1c=K1c, K2c=K2c, covs=covs, Acovs=Acovs, trait_covar_type=trait_covar_type, rank=rank)
        K1c = vc.getTraitCovar(0)
        K2c = vc.getTraitCovar(1)
    else:
        assert K1c.shape[0]==P, 'K1c: dimensions dismatch'
        assert K1c.shape[1]==P, 'K1c: dimensions dismatch'
        assert K2c.shape[0]==P, 'K2c: dimensions dismatch'
        assert K2c.shape[1]==P, 'K2c: dimensions dismatch'

    #2. run kroneckerLMM for null model
    lmm = limix.deprecated.CKroneckerLMM()
    lmm.setK1r(K1r)
    lmm.setK1c(K1c)
    lmm.setK2r(K2r)
    lmm.setK2c(K2c)
    lmm.setSNPs(snps)
    #add covariates
    for ic  in xrange(len(Acovs)):
        lmm.addCovariates(covs[ic],Acovs[ic])
    lmm.setPheno(phenos)

    #delta serch on alt. model?
    if searchDelta:
        lmm.setNumIntervalsAlt(NumIntervalsDeltaAlt)
        lmm.setNumIntervals0_inter(NumIntervalsDeltaAlt)
    else:
        lmm.setNumIntervalsAlt(0)
        lmm.setNumIntervals0_inter(0)


    lmm.setNumIntervals0(NumIntervalsDelta0)
    #add SNP design
    lmm.setSNPcoldesign0_inter(Asnps0[0])
    for iA in xrange(len(Asnps1)):
        lmm.setSNPcoldesign(Asnps1[iA])
        lmm.process()

        pvAlt[iA,:] = lmm.getPv()[0]
        pv[iA,:] = lmm.getPv()[1]
        pv0 = lmm.getPv()[2][np.newaxis,:]
    if return_lmm:
        return pv,pv0,pvAlt,lmm
    else:
        return pv,pv0,pvAlt


def test_interaction_lmm(snps,pheno,Inter,Inter0=None,covs=None,K=None,test='lrt'):
    """
    I-variate fixed effects interaction test for phenotype specific SNP effects

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        pheno:  [N x 1] np.array of 1 phenotype for N individuals
        Inter:  [N x I] np.array of I interaction variables to be tested for N
                        individuals (optional). If not provided, only the SNP is
                        included in the null model.
        Inter0: [N x I0] np.array of I0 interaction variables to be included in the
                         background model when testing for interaction with Inter
        covs:   [N x D] np.array of D covariates for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        test:    'lrt' for likelihood ratio test (default) or 'f' for F-test

    Returns:
        limix LMM object
    """
    N=snps.shape[0]
    if covs is None:
        covs = np.ones((N,1))
    if K is None:
        K = np.eye(N)
    if Inter0 is None:
        Inter0=np.ones([N,1])
    assert (pheno.shape[0]==N and K.shape[0]==N and K.shape[1]==N and covs.shape[0]==N and Inter0.shape[0]==N and Inter.shape[0]==N), "shapes missmatch"
    lmi = limix.deprecated.CInteractLMM()
    lmi.setK(K)
    lmi.setSNPs(snps)
    lmi.setPheno(pheno)
    lmi.setCovs(covs)
    lmi.setInter0(Inter0)
    lmi.setInter(Inter)
    if test=='lrt':
        lmi.setTestStatistics(lmi.TEST_LRT)
    elif test=='f':
        lmi.setTestStatistics(lmi.TEST_F)
    else:
        print test
        raise NotImplementedError("only f or lrt are implemented")
    lmi.process()
    return lmi


""" MULTI LOCUS MODEL """


def forward_lmm(snps,pheno,K=None,covs=None,qvalues=False,threshold=5e-8,maxiter=2,test='lrt',verbose=None,**kw_args):
    """
    univariate fixed effects test with forward selection

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        pheno:  [N x 1] np.array of 1 phenotype for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        covs:   [N x D] np.array of D covariates for N individuals
        threshold:      (float) P-value thrashold for inclusion in forward selection (default 5e-8)
        maxiter:        (int) maximum number of interaction scans. First scan is
                        without inclusion, so maxiter-1 inclusions can be performed. (default 2)
        test:           'lrt' for likelihood ratio test (default) or 'f' for F-test
        verbose: print verbose output? (False)

    Returns:
        lm:     limix LMM object
        RV:     dictionary
                RV['iadded']:   array of indices of SNPs included in order of inclusion
                RV['pvadded']:  array of Pvalues obtained by the included SNPs in iteration
                                before inclusion
                RV['pvall']:    [Nadded x S] np.array of Pvalues for all iterations
    """
    verbose = dlimix.getVerbose(verbose)

    if K is None:
        K=np.eye(snps.shape[0])
    if covs is None:
        covs = np.ones((snps.shape[0],1))
    #assert single trait
    assert pheno.shape[1]==1, 'forward_lmm only supports single phenotypes'

    lm = test_lmm(snps,pheno,K=K,covs=covs,test=test,**kw_args)
    pvall = []
    pv = lm.getPv().ravel()
    #hack to avoid issues with degenerate pv
    pv[sp.isnan(pv)] = 1
    pvall.append(pv)
    imin= pv.argmin()
    niter = 1
    #start stuff
    iadded = []
    pvadded = []
    qvadded = []
    if qvalues:
        assert pv.shape[0]==1, "This is untested with the fdr package. pv.shape[0]==1 failed"
        qvall = []
        qv  = FDR.qvalues(pv)
        qvall.append(qv)
        score=qv.min()
    else:
        score=pv.min()
    while (score<threshold) and niter<maxiter:
        t0=time.time()
        iadded.append(imin)
        pvadded.append(pv[imin])
        if qvalues:
            qvadded.append(qv[0,imin])
        covs=np.concatenate((covs,snps[:,imin:(imin+1)]),1)
        lm.setCovs(covs)
        lm.process()
        pv = lm.getPv().ravel()
        pv[sp.isnan(pv)] = 1
        pvall.append(pv)
        imin= pv.argmin()
        if qvalues:
            qv = FDR.qvalues(pv)
            qvall[niter:niter+1,:] = qv
            score = qv.min()
        else:
            score = pv.min()
        t1=time.time()
        if verbose:
            print ("finished GWAS testing in %.2f seconds" %(t1-t0))
        niter=niter+1
    RV = {}
    RV['iadded']  = iadded
    RV['pvadded'] = pvadded
    RV['pvall']   = np.array(pvall)
    if qvalues:
        RV['qvall'] = np.array(qvall)
        RV['qvadded'] = qvadded
    return lm,RV


#TOOD: use **kw_args to forward params.. see below
def forward_lmm_kronecker(snps,phenos,Asnps=None,Acond=None,K1r=None,K1c=None,K2r=None,K2c=None,covs=None,Acovs=None,threshold=5e-8,maxiter=2,qvalues=False, update_covariances = False,verbose=None,**kw_args):
    """
    Kronecker fixed effects test with forward selection

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        pheno:  [N x P] np.array of 1 phenotype for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        covs:   [N x D] np.array of D covariates for N individuals
        threshold:      (float) P-value thrashold for inclusion in forward selection (default 5e-8)
        maxiter:        (int) maximum number of interaction scans. First scan is
                        without inclusion, so maxiter-1 inclusions can be performed. (default 2)
        qvalues:        Use q-value threshold and return q-values in addition (default False)
        update_covar:   Boolean indicator if covariances should be re-estimated after each forward step (default False)

    Returns:
        lm:             lmix LMMi object
        resultStruct with elements:
            iadded:         array of indices of SNPs included in order of inclusion
            pvadded:        array of Pvalues obtained by the included SNPs in iteration
                            before inclusion
            pvall:     [Nadded x S] np.array of Pvalues for all iterations.
        Optional:      corresponding q-values
            qvadded
            qvall
    """
    verbose = dlimix.getVerbose(verbose)
    #0. checks
    N  = phenos.shape[0]
    P  = phenos.shape[1]

    if K1r==None:
        K1r = np.dot(snps,snps.T)
    else:
        assert K1r.shape[0]==N, 'K1r: dimensions dismatch'
        assert K1r.shape[1]==N, 'K1r: dimensions dismatch'

    if K2r==None:
        K2r = np.eye(N)
    else:
        assert K2r.shape[0]==N, 'K2r: dimensions dismatch'
        assert K2r.shape[1]==N, 'K2r: dimensions dismatch'

    covs,Acovs = _updateKronCovs(covs,Acovs,N,P)

    if Asnps is None:
        Asnps = [np.ones([1,P])]
    if (type(Asnps)!=list):
        Asnps = [Asnps]
    assert len(Asnps)>0, "need at least one Snp design matrix"

    if Acond is None:
        Acond = Asnps
    if (type(Acond)!=list):
        Acond = [Acond]
    assert len(Acond)>0, "need at least one Snp design matrix"

    #1. run GP model to infer suitable covariance structure
    if K1c==None or K2c==None:
        vc = _estimateKronCovariances(phenos=phenos, K1r=K1r, K2r=K2r, K1c=K1c, K2c=K2c, covs=covs, Acovs=Acovs, **kw_args)
        K1c = vc.getTraitCovar(0)
        K2c = vc.getTraitCovar(1)
    else:
        vc = None
        assert K1c.shape[0]==P, 'K1c: dimensions dismatch'
        assert K1c.shape[1]==P, 'K1c: dimensions dismatch'
        assert K2c.shape[0]==P, 'K2c: dimensions dismatch'
        assert K2c.shape[1]==P, 'K2c: dimensions dismatch'
    t0 = time.time()
    lm,pv = test_lmm_kronecker(snps=snps,phenos=phenos,Asnps=Asnps,K1r=K1r,K2r=K2r,K1c=K1c,K2c=K2c,covs=covs,Acovs=Acovs)

    #get pv
    #start stuff
    iadded = []
    pvadded = []
    qvadded = []
    time_el = []
    pvall = []
    qvall = None
    t1=time.time()
    if verbose:
        print ("finished GWAS testing in %.2f seconds" %(t1-t0))
    time_el.append(t1-t0)
    pvall.append(pv)
    imin= np.unravel_index(pv.argmin(),pv.shape)
    score=pv[imin].min()
    niter = 1
    if qvalues:
        assert pv.shape[0]==1, "This is untested with the fdr package. pv.shape[0]==1 failed"
        qvall = []
        qv  = FDR.qvalues(pv)
        qvall.append(qv)
        score=qv[imin]
    #loop:
    while (score<threshold) and niter<maxiter:
        t0=time.time()
        pvadded.append(pv[imin])
        iadded.append(imin)
        if qvalues:
            qvadded.append(qv[imin])
        if update_covariances and vc is not None:
            vc.addFixedTerm(snps[:,imin[1]:(imin[1]+1)],Acond[imin[0]])
            vc.setScales()#CL: don't know what this does, but findLocalOptima crashes becahuse vc.noisPos=None
            vc.findLocalOptima(fast=True)
            K1c = vc.getTraitCovar(0)
            K2c = vc.getTraitCovar(1)
            lm.setK1c(K1c)
            lm.setK2c(K2c)
        lm.addCovariates(snps[:,imin[1]:(imin[1]+1)],Acond[imin[0]])
        for i in xrange(len(Asnps)):
            #add SNP design
            lm.setSNPcoldesign(Asnps[i])
            lm.process()
            pv[i,:] = lm.getPv()[0]
        pvall.append(pv.ravel())
        imin= np.unravel_index(pv.argmin(),pv.shape)
        if qvalues:
            qv = FDR.qvalues(pv)
            qvall[niter:niter+1,:] = qv
            score = qv[imin].min()
        else:
            score = pv[imin].min()
        t1=time.time()
        if verbose:
            print ("finished GWAS testing in %.2f seconds" %(t1-t0))
        time_el.append(t1-t0)
        niter=niter+1
    RV = {}
    RV['iadded']  = iadded
    RV['pvadded'] = pvadded
    RV['pvall']   = np.array(pvall)
    RV['time_el'] = time_el
    if qvalues:
        RV['qvall'] = qvall
        RV['qvadded'] = qvadded
    return lm,RV



""" INTERNAL """
def _estimateKronCovariances(phenos,K1r=None,K1c=None,K2r=None,K2c=None,covs=None,Acovs=None,trait_covar_type='lowrank_diag',rank=1,lambd=None,verbose=True,init_method='random',old_opt=True):
	"""
	estimates the background covariance model before testing

	Args:
		phenos: [N x P] np.array of P phenotypes for N individuals
		K1r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
						If not provided, then linear regression analysis is performed
		K1c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
						If not provided, then linear regression analysis is performed
		K2r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
						If not provided, then linear regression analysis is performed
		K2c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
						If not provided, then linear regression analysis is performed
		covs:           list of np.arrays holding covariates. Each covs[i] has one corresponding Acovs[i]
		Acovs:          list of np.arrays holding the phenotype design matrices for covariates.
						Each covs[i] has one corresponding Acovs[i].
		trait_covar_type:     type of covaraince to use. Default 'freeform'. possible values are
						'freeform': free form optimization,
						'fixed': use a fixed matrix specified in covar_K0,
						'diag': optimize a diagonal matrix,
						'lowrank': optimize a low rank matrix. The rank of the lowrank part is specified in the variable rank,
						'lowrank_id': optimize a low rank matrix plus the weight of a constant diagonal matrix. The rank of the lowrank part is specified in the variable rank,
						'lowrank_diag': optimize a low rank matrix plus a free diagonal matrix. The rank of the lowrank part is specified in the variable rank,
						'block': optimize the weight of a constant P x P block matrix of ones,
						'block_id': optimize the weight of a constant P x P block matrix of ones plus the weight of a constant diagonal matrix,
						'block_diag': optimize the weight of a constant P x P block matrix of ones plus a free diagonal matrix,
		rank:           rank of a possible lowrank component (default 1)

	Returns:
		VarianceDecomposition object
	"""
	print ".. Training the backgrond covariance with a GP model"
	vc = VAR.VarianceDecomposition(phenos)
	if K1r is not None:
		vc.addRandomEffect(K1r,trait_covar_type=trait_covar_type,rank=rank)
	if K2r is not None:
		#TODO: fix this; forces second term to be the noise covariance
		vc.addRandomEffect(is_noise=True,K=K2r,trait_covar_type=trait_covar_type,rank=rank)
	for ic  in xrange(len(Acovs)):
		vc.addFixedEffect(covs[ic],Acovs[ic])
	start = time.time()
	if old_opt:
		conv = vc.optimize(fast=True)
	elif lambd is not None:
		conv = vc.optimize(init_method=init_method,verbose=verbose,lambd=lambd)
	else:
		conv = vc.optimize(init_method=init_method,verbose=verbose)
	assert conv, "Variance Decomposition has not converged"
	time_el = time.time()-start
	print "Background model trained in %.2f s" % time_el
	return vc

def _updateKronCovs(covs,Acovs,N,P):
    """
    make sure that covs and Acovs are lists
    """
    if (covs is None) and (Acovs is None):
        covs = [np.ones([N,1])]
        Acovs = [np.eye(P)]

    if Acovs is None or covs is None:
        raise Exception("Either Acovs or covs is None, while the other isn't")

    if (type(Acovs)!=list) and (type(covs)!=list):
        Acovs= [Acovs]
        covs = [covs]
    if (type(covs)!=list) or (type(Acovs)!=list) or (len(covs)!=len(Acovs)):
        raise Exception("Either Acovs or covs is not a list or they missmatch in length")
    return covs, Acovs


""" DEPRECATED AND/OR NOT USED"""

def test_interaction_kronecker_deprecated(snps,phenos,covs=None,Acovs=None,Asnps1=None,Asnps0=None,K1r=None,K1c=None,K2r=None,K2c=None,trait_covar_type='lowrank_diag',rank=1,searchDelta=False):
    """
    I-variate fixed effects interaction test for phenotype specific SNP effects.
    (Runs multiple likelihood ratio tests and computes the P-values in python from the likelihood ratios)

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        phenos: [N x P] np.array of P phenotypes for N individuals
        covs:           list of np.arrays holding covariates. Each covs[i] has one corresponding Acovs[i]
        Acovs:          list of np.arrays holding the phenotype design matrices for covariates.
                        Each covs[i] has one corresponding Acovs[i].
        Asnps1:         list of np.arrays of I interaction variables to be tested for N
                        individuals. Note that it is assumed that Asnps0 is already included.
                        If not provided, the alternative model will be the independent model
        Asnps0:         single np.array of I0 interaction variables to be included in the
                        background model when testing for interaction with Inters
        K1r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K1c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2r:    [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        K2c:    [P x P] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        trait_covar_type:     type of covaraince to use. Default 'freeform'. possible values are
                        'freeform': free form optimization,
                        'fixed': use a fixed matrix specified in covar_K0,
                        'diag': optimize a diagonal matrix,
                        'lowrank': optimize a low rank matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_id': optimize a low rank matrix plus the weight of a constant diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'lowrank_diag': optimize a low rank matrix plus a free diagonal matrix. The rank of the lowrank part is specified in the variable rank,
                        'block': optimize the weight of a constant P x P block matrix of ones,
                        'block_id': optimize the weight of a constant P x P block matrix of ones plus the weight of a constant diagonal matrix,
                        'block_diag': optimize the weight of a constant P x P block matrix of ones plus a free diagonal matrix,
        rank:           rank of a possible lowrank component (default 1)
        searchDelta:    Boolean indicator if delta is optimized during SNP testing (default False)

    Returns:
        pv:     P-values of the interaction test
        lrt0:   log likelihood ratio statistics of the null model
        pv0:    P-values of the null model
        lrt:    log likelihood ratio statistics of the interaction test
        lrtAlt: log likelihood ratio statistics of the alternative model
        pvAlt:  P-values of the alternative model
    """
    S=snps.shape[1]
    #0. checks
    N  = phenos.shape[0]
    P  = phenos.shape[1]

    if K1r==None:
        K1r = np.dot(snps,snps.T)
    else:
        assert K1r.shape[0]==N, 'K1r: dimensions dismatch'
        assert K1r.shape[1]==N, 'K1r: dimensions dismatch'

    if K2r==None:
        K2r = np.eye(N)
    else:
        assert K2r.shape[0]==N, 'K2r: dimensions dismatch'
        assert K2r.shape[1]==N, 'K2r: dimensions dismatch'

    covs,Acovs = _updateKronCovs(covs,Acovs,N,P)

    #Asnps can be several designs
    if (Asnps0 is None):
        Asnps0 = [np.ones([1,P])]
    if Asnps1 is None:
        Asnps1 = [np.eye([P])]
    if (type(Asnps0)!=list):
        Asnps0 = [Asnps0]
    if (type(Asnps1)!=list):
        Asnps1 = [Asnps1]
    assert (len(Asnps0)==1) and (len(Asnps1)>0), "need at least one Snp design matrix for null and alt model"

    #one row per column design matrix
    pv = np.zeros((len(Asnps1),snps.shape[1]))
    lrt = np.zeros((len(Asnps1),snps.shape[1]))
    pvAlt = np.zeros((len(Asnps1),snps.shape[1]))
    lrtAlt = np.zeros((len(Asnps1),snps.shape[1]))

    #1. run GP model to infer suitable covariance structure
    if K1c==None or K2c==None:
        vc = _estimateKronCovariances(phenos=phenos, K1r=K1r, K2r=K2r, K1c=K1c, K2c=K2c, covs=covs, Acovs=Acovs, trait_covar_type=trait_covar_type, rank=rank)
        K1c = vc.getTraitCovar(0)
        K2c = vc.getTraitCovar(1)
    else:
        assert K1c.shape[0]==P, 'K1c: dimensions dismatch'
        assert K1c.shape[1]==P, 'K1c: dimensions dismatch'
        assert K2c.shape[0]==P, 'K2c: dimensions dismatch'
        assert K2c.shape[1]==P, 'K2c: dimensions dismatch'

    #2. run kroneckerLMM for null model
    lmm = limix.deprecated.CKroneckerLMM()
    lmm.setK1r(K1r)
    lmm.setK1c(K1c)
    lmm.setK2r(K2r)
    lmm.setK2c(K2c)
    lmm.setSNPs(snps)
    #add covariates
    for ic  in xrange(len(Acovs)):
        lmm.addCovariates(covs[ic],Acovs[ic])
    lmm.setPheno(phenos)
    if searchDelta:      lmm.setNumIntervalsAlt(100)
    else:                   lmm.setNumIntervalsAlt(0)
    lmm.setNumIntervals0(100)
    #add SNP design
    lmm.setSNPcoldesign(Asnps0[0])
    lmm.process()
    dof0 = Asnps0[0].shape[0]
    pv0 = lmm.getPv()
    lrt0 = st.chi2.isf(pv0,dof0)
    for iA in xrange(len(Asnps1)):
        dof1 = Asnps1[iA].shape[0]
        dof = dof1-dof0
        lmm.setSNPcoldesign(Asnps1[iA])
        lmm.process()
        pvAlt[iA,:] = lmm.getPv()[0]
        lrtAlt[iA,:] = st.chi2.isf(pvAlt[iA,:],dof1)
        lrt[iA,:] = lrtAlt[iA,:] - lrt0[0] # Don't need the likelihood ratios, as null model is the same between the two models
        pv[iA,:] = st.chi2.sf(lrt[iA,:],dof)
    return pv,lrt0,pv0,lrt,lrtAlt,pvAlt

#TODO: we need to fix. THis does not work as interact_GxE is not existing
#I vote we also use **kw_args to forward parameters to interact_Gxe?
def test_interaction_GxG(pheno,snps1,snps2=None,K=None,covs=None,test='lrt'):
    """
    Epistasis test between two sets of SNPs

    Args:
        pheno:  [N x 1] np.array of 1 phenotype for N individuals
        snps1:  [N x S1] np.array of S1 SNPs for N individuals
        snps2:  [N x S2] np.array of S2 SNPs for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        covs:   [N x D] np.array of D covariates for N individuals
        test:    'lrt' for likelihood ratio test (default) or 'f' for F-test

    Returns:
        pv:     [S2 x S1] np.array of P values for epistasis tests beten all SNPs in
                snps1 and snps2
    """
    if K is None:
        K=np.eye(N)
    N=snps1.shape[0]
    if snps2 is None:
        snps2 = snps1
    return test_interaction_GxE_1dof(snps=snps1,pheno=pheno,env=snps2,covs=covs,K=K,test=test)


def test_interaction_GxE_1dof(snps,pheno,env,K=None,covs=None, test='lrt',verbose=None):
    """
    Univariate GxE fixed effects interaction linear mixed model test for all
    pairs of SNPs and environmental variables.

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals
        pheno:  [N x 1] np.array of 1 phenotype for N individuals
        env:    [N x E] np.array of E environmental variables for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        covs:   [N x D] np.array of D covariates for N individuals
        test:    'lrt' for likelihood ratio test (default) or 'f' for F-test
        verbose: print verbose output? (False)

    Returns:
        pv:     [E x S] np.array of P values for interaction tests between all
                E environmental variables and all S SNPs
    """
    verbose = dlimix.getVerbose(verbose)
    N=snps.shape[0]
    if K is None:
        K=np.eye(N)
    if covs is None:
        covs = np.ones((N,1))
    assert (env.shape[0]==N and pheno.shape[0]==N and K.shape[0]==N and K.shape[1]==N and covs.shape[0]==N), "shapes missmatch"
    Inter0 = np.ones((N,1))
    pv = np.zeros((env.shape[1],snps.shape[1]))
    if verbose:
        print ("starting %i interaction scans for %i SNPs each." % (env.shape[1], snps.shape[1]))
    t0=time.time()
    for i in xrange(env.shape[1]):
        t0_i = time.time()
        cov_i = np.concatenate((covs,env[:,i:(i+1)]),1)
        lm_i = test_interaction_lmm(snps=snps,pheno=pheno,covs=cov_i,Inter=env[:,i:(i+1)],Inter0=Inter0,test=test)
        pv[i,:]=lm_i.getPv()[0,:]
        t1_i = time.time()
        if verbose:
            print ("Finished %i out of %i interaction scans in %.2f seconds."%((i+1),env.shape[1],(t1_i-t0_i)))
    t1 = time.time()
    print ("-----------------------------------------------------------\nFinished all %i interaction scans in %.2f seconds."%(env.shape[1],(t1-t0)))
    return pv


def phenSpecificEffects(snps,pheno1,pheno2,K=None,covs=None,test='lrt'):
    """
    Univariate fixed effects interaction test for phenotype specific SNP effects

    Args:
        snps:   [N x S] np.array of S SNPs for N individuals (test SNPs)
        pheno1: [N x 1] np.array of 1 phenotype for N individuals
        pheno2: [N x 1] np.array of 1 phenotype for N individuals
        K:      [N x N] np.array of LMM-covariance/kinship koefficients (optional)
                        If not provided, then linear regression analysis is performed
        covs:   [N x D] np.array of D covariates for N individuals
        test:    'lrt' for likelihood ratio test (default) or 'f' for F-test

    Returns:
        limix LMM object
    """
    N=snps.shape[0]
    if K is None:
        K=np.eye(N)
    assert (pheno1.shape[1]==pheno2.shape[1]), "Only consider equal number of phenotype dimensions"
    if covs is None:
        covs = np.ones(N,1)
    assert (pheno1.shape[1]==1 and pheno2.shape[1]==1 and pheno1.shape[0]==N and pheno2.shape[0]==N and K.shape[0]==N and K.shape[1]==N and covs.shape[0]==N), "shapes missmatch"
    Inter = np.zeros((N*2,1))
    Inter[0:N,0]=1
    Inter0 = np.ones((N*2,1))
    Yinter=np.concatenate((pheno1,pheno2),0)
    Xinter = np.tile(snps,(2,1))
    Covitner= np.tile(covs(2,1))
    lm = test_interaction_lmm(snps=Xinter,pheno=Yinter,covs=Covinter,Inter=Inter,Inter0=Inter0,test=test)
    return lm



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import scipy as sp

# A simple function to calculat the minor allele frequency
# Here, we assume that M is either [0,1] or [0,1,2]
# The same principles applies if we have genotype likelihoods rather than hard thresholds

def calc_AF(M,major=0,minor=2):
    """calculate minor allelel frequency, by default assuming that minor==2"""
    if minor==2:
        Nhet   = (M==0).sum(axis=0)
        Nmajor = 2*(M==0).sum(axis=0)
        Nminor = 2*(M==2).sum(axis=0)
        af  = Nminor/sp.double(2*M.shape[0])
    else:
        Nmajor = (M==0).sum(axis=0)
        Nminor = (M==1).sum(axis=0)
        af  = Nminor/sp.double(1*M.shape[0])
    RV = {}
    RV['af'] = af
    RV['Nmajor'] = Nmajor
    RV['Nminor'] = Nminor
    return RV

def calc_LD(M,pos,i_start=[0],max_dist=1000000):
    """calculate linkage disequilibrium correlations:
    M: genotype matrix
    pos: position vector
    i_start: index to start from for LD calculation
    dist: distance
    """
    RV = []
    DIST = []
    for start in i_start:
        pos0 = pos[start]
        v0  = M[:,start]
        Iselect = sp.nonzero(sp.absolute(pos-pos0)<=max_dist)[0]
        rv = sp.zeros(len(Iselect))
        for i in xrange(len(Iselect)):
            rv[i] = (sp.corrcoef(v0,M[:,Iselect[i]])[0,1])**2
        #sort by distance
        dist = sp.absolute(pos[Iselect]-pos0)
        RV.extend(rv)
        DIST.extend(dist)
    RV = sp.array(RV)
    DIST = sp.array(DIST)
    II = DIST.argsort()
    return [DIST[II],RV[II]]


import numpy as np
import scipy as sp
import scipy.linalg as LA
from covar_base import Covariance
from limix.core.type.cached import cached
import pdb

import logging as LG

class FreeFormCov(Covariance):
    """
    General semi-definite positive matrix with no contraints.
    A free-form covariance matrix of dimension d has 1/2 * d * (d + 1) params
    """
    def __init__(self, dim, jitter=1e-4):
        """
        Args:
            dim:        dimension of the free-form covariance
            jitter:     extent of diagonal offset which is added for numerical stability
                        (default value: 1e-4)
        """
        Covariance.__init__(self, dim)
        self._K_act = True
        self._calcNumberParams()
        self.dim = dim
        self.params = sp.zeros(self.n_params)
        self.idx_r, self.idx_c = sp.tril_indices(self.dim)
        self.set_jitter(jitter)

    #####################
    # Properties
    #####################
    @property
    def variance(self):
        return self.K().diagonal()

    @property
    def correlation(self):
        R = self.K().copy()
        inv_diag = 1./sp.sqrt(R.diagonal())[:,sp.newaxis]
        R *= inv_diag
        R *= inv_diag.T
        return R

    @property
    def variance_ste(self):
        if self.getFIinv() is None:
            R = None
        else:
            R = self.K_ste().diagonal()
        # IN A VARIANCE / CORRELATION PARAMETRIZATION
        #if self.getFIinv() is None:
        #   R = None
        #else:
        #   R = sp.sqrt(self.getFIinv().diagonal()[:self.dim])
        return R

    @property
    def correlation_ste(self):
        if self.getFIinv() is None:
            R = None
        else:
            idx_M = sp.zeros((self.dim,self.dim))
            idx_M[sp.tril_indices(self.dim)] = sp.arange( int( 0.5 * self.dim * (self.dim + 1) ) )
            R = sp.zeros(idx_M)
            for i in range(self.dim):
                for j in range(0,self.dim):
                    ij = idx_M[i,j] # index of cov_ij_ste from fisher
                    ii = idx_M[i,i] # index of cov_ii_ste from fisher
                    jj = idx_M[j,j] # index of cov_jj_ste from fisher
                    #TODO: complete

        # IN A VARIANCE / CORRELATION PARAMETRIZATION
        #if self.getFIinv() is None:
        #    R = None
        #else:
        #    R = sp.zeros((self.dim, self.dim))
        #    R[sp.tril_indices(self.dim, k = -1)] = sp.sqrt(self.getFIinv().diagonal()[self.dim:])
        #    R += R.T
        return R

    #####################
    # Activation handling
    #####################
    @property
    def act_K(self):
        return self._K_act

    @act_K.setter
    def act_K(self, act):
        self._K_act = bool(act)
        self._notify()

    #####################
    # Params handling
    #####################
    def setParams(self, params):
        if not self._K_act and len(params) > 0:
            raise ValueError("Trying to set a parameter via setParams that "
                             "is not active.")
        if self._K_act:
            self.params[:] = params
            self.clear_all()

    def getParams(self):
        if not self._K_act:
            return np.array([])
        return self.params

    def getNumberParams(self):
        return int(self._K_act) * self.n_params

    def _calcNumberParams(self):
        self.n_params = int(0.5*self.dim*(self.dim+1))

    def set_jitter(self,value):
        self.jitter = value

    def setCovariance(self,cov):
        """ set hyperparameters from given covariance """
        chol = LA.cholesky(cov,lower=True)
        params = chol[sp.tril_indices(self.dim)]
        self.setParams(params)

    #####################
    # Cached
    #####################
    @cached('covar_base')
    def K(self):
        RV = sp.dot(self.L(),self.L().T)+self.jitter*sp.eye(self.dim)
        return RV

    @cached('covar_base')
    def K_grad_i(self,i):
        if not self._K_act:
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")
        RV = sp.dot(self.L(),self.Lgrad(i).T)+sp.dot(self.Lgrad(i),self.L(i).T)
        return RV

    @cached
    def K_hess_i_j(self, i, j):
        if not self._K_act:
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")
        RV = sp.dot(self.Lgrad(i),self.Lgrad(j).T)
        RV+= RV.T
        return RV

    def K_ste(self):
        if self.getFIinv() is None:
            R = None
        else:
            R = sp.zeros((self.dim, self.dim))
            R[sp.tril_indices(self.dim)] = sp.sqrt(self.getFIinv().diagonal())
            # symmetrize
            R = R + R.T - sp.diag(R.diagonal())
        return R

    ####################
    # Interpretable Params
    ####################
    def getInterParams(self):
        # VARIANCE + CORRELATIONS
        #R1 = self.variance
        #R2 = self.correlation[sp.tril_indices(self.dim, k = -1)]
        #R = sp.concatenate([R1,R2])

        # COVARIANCES
        R = self.K()[sp.tril_indices(self.dim)]
        return R

    # DERIVARIVE WITH RESPECT TO COVARIANCES
    def K_grad_interParam_i(self, i):
        ix, iy = sp.tril_indices(self.dim)
        ix = ix[i]
        iy = iy[i]
        R = sp.zeros((self.dim,self.dim))
        R[ix, iy] = R[iy, ix] = 1
        return R

    # DERIVARIVE WITH RESPECT TO VARIANCES AND CORRELATIONS
    #def K_grad_interParam_i(self, i):
    #    if i < self.dim:
    #        # derivative with respect to the variance
    #        R = sp.zeros((self.dim,self.dim))
    #        R[i,:] = self.K()[i,:] / (2 * self.variance[i])
    #        R += R.T
    #    else:
    #        # derivarice with respect to a correlation
    #        ## 1. take the corresponding off diagonal element
    #        ix, iy = sp.tril_indices(self.dim, k = -1)
    #        ix = ix[i - self.dim]
    #        iy = iy[i - self.dim]
    #        ## 2. fill it with sqrt(var * var)
    #        R = sp.zeros((self.dim,self.dim))
    #        R[ix,iy] = R[iy,ix] = sp.sqrt(self.variance[ix] * self.variance[iy])
    #    return R

    ######################
    # Private functions
    ######################
    @cached('covar_base')
    def L(self):
        R = sp.zeros((self.dim, self.dim))
        R[(self.idx_r, self.idx_c)] = self.params
        return R

    @cached
    def Lgrad(self, i):
        R = sp.zeros((self.dim, self.dim))
        R[self.idx_r[i], self.idx_c[i]] = 1
        return R

if __name__ == '__main__':
    n = 2
    cov = FreeFormCov(n)
    print cov.K()
    print cov.K_grad_i(0)



# Copyright(c) 2014, The LIMIX developers (Christoph Lippert, Paolo Francesco Casale, Oliver Stegle)
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
"""preprocessing functions"""

import scipy as SP
import scipy.special as special
import scipy.stats as st
import pdb


def variance_K(K, verbose=False):
    """estimate the variance explained by K"""
    c = SP.sum((SP.eye(len(K)) - (1.0 / len(K)) * SP.ones(K.shape)) * SP.array(K))
    scalar = (len(K) - 1) / c
    return 1.0/scalar


def scale_K(K, verbose=False,trace_method=True):
    """scale covariance K such that it explains unit variance
    trace_method: standardize to unit trace (deafault: True)
    """
    if trace_method:
        scalar=1.0/(K.diagonal().mean())
    else:
        c = SP.sum((SP.eye(len(K)) - (1.0 / len(K)) * SP.ones(K.shape)) * SP.array(K))
        scalar = (len(K) - 1) / c
    if verbose:
        print 'Kinship scaled by: %0.4f' % scalar
    K = K * scalar
    return K


def standardize(Y,in_place=False):
    """
    standardize Y in a way that is robust to missing values
    in_place: create a copy or carry out inplace opreations?
    """
    if in_place:
        YY = Y
    else:
        YY = Y.copy()
    for i in xrange(YY.shape[1]):
        Iok = ~SP.isnan(YY[:,i])
        Ym = YY[Iok,i].mean()
        YY[:,i]-=Ym
        Ys = YY[Iok,i].std()
        YY[:,i]/=Ys
    return YY
    

def rankStandardizeNormal(X):
	"""
	Gaussianize X: [samples x phenotypes]
	- each phentoype is converted to ranks and transformed back to normal using the inverse CDF
	"""
	Is = X.argsort(axis=0)
	RV = SP.zeros_like(X)
	rank = SP.zeros_like(X)
	for i in xrange(X.shape[1]):
		x =  X[:,i]
		i_nan = SP.isnan(x)
		if 0:
			Is = x.argsort()
			rank = SP.zeros_like(x)
			rank[Is] = SP.arange(X.shape[0])
			#add one to ensure nothing = 0
			rank +=1
		else:
			rank = st.rankdata(x[~i_nan])
		#devide by (N+1) which yields uniform [0,1]
		rank /= ((~i_nan).sum()+1)
		#apply inverse gaussian cdf
		RV[~i_nan,i] = SP.sqrt(2) * special.erfinv(2*rank-1)
		RV[i_nan,i] = x[i_nan]
	return RV


def boxcox(X):
	"""
    Gaussianize X using the Box-Cox transformation: [samples x phenotypes]

    - each phentoype is brought to a positive schale, by first subtracting the minimum value and adding 1.
    - Then each phenotype transformed by the boxcox transformation
	"""
	X_transformed = SP.zeros_like(X)
	maxlog = SP.zeros(X.shape[1])
	for i in xrange(X.shape[1]):
		i_nan = SP.isnan(X[:,i])
		values = X[~i_nan,i]
		X_transformed[i_nan,i] = X[i_nan,i]
		X_transformed[~i_nan,i], maxlog[i] = st.boxcox(values-values.min()+1.0)
	return X_transformed, maxlog





from scipy.optimize import fmin_l_bfgs_b as optimize
import pdb
import scipy as SP
import logging as LG
import numpy as np
import time

def param_dict_to_list(dict,skeys=None):
    """convert from param dictionary to list"""
    #sort keys
    RV = SP.concatenate([dict[key].flatten() for key in skeys])
    return RV
    pass

def param_list_to_dict(list,param_struct,skeys):
    """convert from param dictionary to list
    param_struct: structure of parameter array
    """
    RV = []
    i0= 0
    for key in skeys:
        val = param_struct[key]
        shape = SP.array(val)
        np = shape.prod()
        i1 = i0+np
        params = list[i0:i1].reshape(shape)
        RV.append((key,params))
        i0 = i1
    return dict(RV)

def checkgrad(f, fprime, x, *args,**kw_args):
    """
    Analytical gradient calculation using a 3-point method

    """
    LG.debug("Checking gradient ...")
    import numpy as np

    # using machine precision to choose h
    eps = np.finfo(float).eps
    step = np.sqrt(eps)*(x.min())

    # shake things up a bit by taking random steps for each x dimension
    h = step*np.sign(np.random.uniform(-1, 1, x.size))

    f_ph = f(x+h, *args, **kw_args)
    f_mh = f(x-h, *args, **kw_args)
    numerical_gradient = (f_ph - f_mh)/(2*h)
    analytical_gradient = fprime(x, *args, **kw_args)
    ratio = (f_ph - f_mh)/(2*np.dot(h, analytical_gradient))

    h = np.zeros_like(x)
    for i in range(len(x)):
        pdb.set_trace()
	h[i] = step
	f_ph = f(x+h, *args, **kw_args)
	f_mh = f(x-h, *args, **kw_args)
	numerical_gradient = (f_ph - f_mh)/(2*step)
	analytical_gradient = fprime(x, *args, **kw_args)[i]
	ratio = (f_ph - f_mh)/(2*step*analytical_gradient)
	h[i] = 0
        LG.debug("[%d] numerical: %f, analytical: %f, ratio: %f" % (i, numerical_gradient,analytical_gradient,ratio))



def opt_hyper(gpr,Ifilter=None,bounds=None,opts={},*args,**kw_args):
    """
    optimize params

    Input:
    gpr: GP regression class
    params0: dictionary filled with starting hyperparameters
    opts: options for optimizer
    """
    if 'gradcheck' in opts:
        gradcheck = opts['gradcheck']
    else:
        gradcheck = False
    if 'max_iter_opt' in opts:
        max_iter = opts['max_iter_opt']
    else:
        max_iter = 5000
    if 'pgtol' in opts:
        pgtol = opts['pgtol']
    else:
        pgtol = 1e-10

    params0 = gpr.getParams()

    def f(x):
        x_ = X0
        x_[Ifilter_x] = x
        gpr.setParams(param_list_to_dict(x_,param_struct,skeys))
        lml = gpr.LML()
        if SP.isnan(lml):
            lml=1E6
        lml_grad = gpr.LML_grad()
        lml_grad = param_dict_to_list(lml_grad,skeys)
        if (~SP.isfinite(lml_grad)).any():
            idx = (~SP.isfinite(lml_grad))
            lml_grad[idx] = 1E6
        return lml, lml_grad[Ifilter_x]

    skeys = SP.sort(params0.keys())
    param_struct = dict([(name,params0[name].shape) for name in skeys])

    # mask params that should not be optimized
    X0 = param_dict_to_list(params0,skeys)
    if Ifilter is not None:
        Ifilter_x = SP.array(param_dict_to_list(Ifilter,skeys),dtype=bool)
    else:
        Ifilter_x = SP.ones(len(X0),dtype='bool')

    # add bounds if necessary
    if bounds is not None:
        _b = []
        for key in skeys:
            if key in bounds.keys():
                _b.extend(bounds[key])
            else:
                _b.extend([[-SP.inf,+SP.inf]]*params0[key].size)
        bounds = SP.array(_b)
        bounds = bounds[Ifilter_x]

    LG.info('Starting optimization ...')
    t = time.time()

    x = X0.copy()[Ifilter_x]
    RV = optimize(f,x,maxfun=int(max_iter),pgtol=pgtol,bounds=bounds,**kw_args)
    #RVopt = optimize(f,x,messages=True,maxfun=int(max_iter),pgtol=pgtol,bounds=bounds)
    #LG.info('%s'%OPT.tnc.RCSTRINGS[RVopt[2]])
    #LG.info('Optimization is converged at iteration %d'%RVopt[1])
    #LG.info('Total time: %.2fs'%(time.time()-t))

    info = RV[2]
    conv = info['warnflag']==0

    if gradcheck:
        err = OPT.check_grad(f,df,xopt)
        LG.info("check_grad (post): %.2f"%err)

    return conv,info



def test():
	from unittest import TestLoader
	from unittest import TextTestRunner
	import os
	folder = os.path.dirname(os.path.realpath(__file__))
	suite = TestLoader().discover(folder)
	TextTestRunner(verbosity=2).run(suite)



import unittest
from limix.core.covar import FreeFormCov
from limix.core.mean import MeanKronSum
from limix.core.gp import GP2KronSum
from limix.core.gp import GP
from limix.utils.preprocess import covar_rescale
from limix.utils.check_grad import mcheck_grad

import numpy as np
import scipy as sp
import pdb


class TestGPBase(unittest.TestCase):
    def setUp(self):
        np.random.seed(1)

        # define phenotype
        N = 200
        P = 2
        self.Y = sp.randn(N, P)
        # define fixed effects
        self.F = []; self.A = []
        self.F.append(1.*(sp.rand(N,2)<0.5))
        self.A.append(sp.eye(P))
        # define row caoriance
        f = 10
        X = 1.*(sp.rand(N, f)<0.2)
        self.R  = covar_rescale(sp.dot(X,X.T))
        self.R += 1e-4 * sp.eye(N)
        # define col covariances
        self.Cg = FreeFormCov(P)
        self.Cn = FreeFormCov(P)
        self.Cg.setCovariance(0.5 * sp.cov(self.Y.T))
        self.Cn.setCovariance(0.5 * sp.cov(self.Y.T))
        # define gp
        self.gp = GP2KronSum(Y=self.Y, F=self.F, A=self.A, Cg=self.Cg,
                             Cn=self.Cn, R=self.R)

    @unittest.expectedFailure
    def test_grad(self):

        gp = self.gp

        def func(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            return gp.LML()

        def grad(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            grad = gp.LML_grad()
            return grad['covar'][i]

        x0 = gp.getParams()['covar']
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=4)

    def test_grad_activation(self):

        gp = self.gp

        self.Cg._K_act = False

        def func(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            return gp.LML()

        def grad(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            grad = gp.LML_grad()
            return grad['covar'][i]

        x0 = gp.getParams()['covar']
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=4)

        self.Cg._K_act = True
        self.Cn._K_act = False

        def func(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            return gp.LML()

        def grad(x, i):
            params = gp.getParams()
            params['covar'] = x
            gp.setParams(params)
            grad = gp.LML_grad()
            return grad['covar'][i]

        x0 = gp.getParams()['covar']
        err = mcheck_grad(func, grad, x0)
        np.testing.assert_almost_equal(err, 0., decimal=4)

    def test_correct_inputs(self):
        np.asarray(None, dtype=float)
        # self.gp = GP2KronSum(Y=self.Y, F=self.F, A=self.A, Cg=self.Cg,
        #                      Cn=self.Cn, R=self.R)

if __name__ == "__main__":
    unittest.main()



import sys
import numpy as np
from covar_base import Covariance
from limix.core.covar import LowRankCov
from limix.core.type.cached import cached
from limix.core.type.exception import TooExpensiveOperationError
from limix.core.utils import my_name
from util import msg_too_expensive_dim
from limix.utils.svd_utils import svd_reduce
import scipy as sp
import scipy.linalg as la
import numpy.linalg as nla
import warnings

import pdb

_MAX_DIM = 5000

class Cov2KronSumLR(Covariance):
    """
    Covariance class for sum of 2 Kronecker products with one low rank term:
        K = Cr \kron GG.T + Cn \kron I
    Notation:
        - dim_c: dimension of col covariances
        - dim_r: dimension of row covariances
        - rank_c: rank of low-rank col covariance
        - rank_r: rank of low-rank row covariance
    """

    def __init__(self, Cn = None, G = None, rank = 1):
        """
        Args:
            Cn:     Limix covariance matrix for Cn (dimension dim_c)
            G:      [dim_r, rank_r] numpy covariance matrix for G
            rank:   rank of column low-rank covariance (default = 1)
        """
        Covariance.__init__(self)
        self._Cr_act = True
        self._Cn_act = True
        self.setColCovars(Cn, rank = rank)
        self.G = G
        self.dim = self.dim_c * self.dim_r
        self._use_to_predict = False

    def G_has_changed(self):
        self.clear_cache('row_cov')
        self._notify('row_cov')
        self.clear_all()

    def col_covs_have_changed(self):
        self.clear_cache('col_cov')
        self._notify('col_cov')
        self.clear_all()

    #####################
    # Properties
    #####################
    @property
    def G(self):
        return self._G

    @property
    def Cr(self):
        return self._Cr

    @property
    def Cn(self):
        return self._Cn

    @property
    def dim_r(self):
        return self._dim_r

    @property
    def dim_c(self):
        return self._dim_c

    @property
    def rank_r(self):
        return self._rank_r

    @property
    def rank_c(self):
        return self._rank_c

    #####################
    # Setters
    #####################
    @G.setter
    def G(self,value):
        assert value is not None, 'G cannot be set to None.'
        self._dim_r = value.shape[0]
        # perform svd on G
        # excludes eigh < 1e-8 and recalcs G
        _value, U, S, V = svd_reduce(value)
        self._rank_r = _value.shape[1]
        self._G  = _value
        self._Ug = U
        self._Sg = S
        self._Vg = V
        self.G_has_changed()

    # normal setter for col covars
    def setColCovars(self, Cn = None, rank = 1):
        assert Cn is not None, 'Cn has to be specified.'
        self._rank_c = rank
        self._dim_c = Cn.dim
        self._Cr = LowRankCov(self._dim_c, rank)
        self._Cn = Cn
        # register
        self._Cr.register(self.col_covs_have_changed)
        self._Cn.register(self.col_covs_have_changed)
        self.col_covs_have_changed()

    #####################
    # Activation handling
    #####################
    @property
    def act_Cr(self):
        return self._Cr_act

    @act_Cr.setter
    def act_Cr(self, act):
        self._Cr_act = bool(act)
        self._notify()

    @property
    def act_Cn(self):
        return self._Cn_act

    @act_Cn.setter
    def act_Cn(self, act):
        self._Cn_act = bool(act)
        self._notify()

    def _actindex2index(self, i):
        nCr = self.Cr.getNumberParams()
        i += nCr * int(not self._Cr_act)
        return i

    def _index2actindex(self, i):
        nCr = self.Cr.getNumberParams()
        i -= nCr * int(not self._Cr_act)
        return i


    #####################
    # Params handling
    #####################
    def setParams(self, params):
        nCr = int(self._Cr_act) * self.Cr.getNumberParams()
        nCn = int(self._Cn_act) * self.Cn.getNumberParams()

        if len(params) != self.getNumberParams():
            raise ValueError("The number of parameters passed to setParams "
                             "differs from the number of active parameters.")
        if self._Cr_act:
            self.Cr.setParams(params[:nCr])
        if self._Cn_act:
            self.Cn.setParams(params[nCr:])

    def getParams(self):
        params = []
        if self._Cr_act:
            params.append(self.Cr.getParams())
        if self._Cn_act:
            params.append(self.Cn.getParams())
        if len(params) == 0:
            return np.array([])
        return sp.concatenate(params)

    def getNumberParams(self):
        return (int(self._Cr_act) * self.Cr.getNumberParams() +
                int(self._Cn_act) * self.Cn.getNumberParams())


    #####################
    # Cached
    #####################
    def Sg(self):
        return self._Sg

    def Ug(self):
        return self._Ug

    def Vg(self):
        return self._Vg

    @cached('row_cov')
    def trSg(self):
        return self.Sg().sum()

    @cached('row_cov')
    def logdetSg(self):
        return sp.log(self.Sg()).sum()

    @cached('row_cov')
    def Wr(self):
        return self.Ug().T

    @cached('col_cov')
    def Lc(self):
        return self.Cn.USi2().T

    @cached
    def Lr(self):
        return self.eye(N)

    @cached('col_cov')
    def Estar(self):
        E = self.Cr.X
        # for a general covariance matrix
        # E = LA.chol(C.K())
        # or based on eigh decomposition if this fails
        return sp.dot(self.Lc(), E)

    @cached('col_cov')
    def Se(self):
        Ue, Seh, Ve = nla.svd(self.Estar(), full_matrices=0)
        self.fill_cache('Ue',Ue)
        self.fill_cache('Ve',Ve)
        return Seh**2

    @cached('col_cov')
    def Ue(self):
        Ue, Seh, Ve = nla.svd(self.Estar(), full_matrices=0)
        self.fill_cache('Se',Seh**2)
        self.fill_cache('Ve',Ve)
        return Ue

    @cached('col_cov')
    def Ve(self):
        Ue, Seh, Ve = nla.svd(self.Estar(), full_matrices=0)
        self.fill_cache('Ue',Ue)
        self.fill_cache('Se',Seh**2)
        return Ve

    @cached('col_cov')
    def Wc(self):
        return self.Ue().T

    @cached(['row_cov', 'col_cov'])
    def SpI(self):
        return sp.kron(1./self.Se(), 1./self.Sg()) + 1

    @cached(['row_cov', 'col_cov'])
    def d(self):
        return 1./self.SpI()

    @cached(['row_cov', 'col_cov'])
    def D(self):
        return self.d().reshape((self.rank_r, self.rank_c), order = 'F')

    @cached(['col_cov'])
    def Ctilde(self, i):
        if i < self.Cr.getNumberParams():
            C = self.Cr.K_grad_i(i)
        else:
            _i = i - self.Cr.getNumberParams()
            C = self.Cn.K_grad_i(_i)
        return sp.dot(self.Lc(), sp.dot(C, self.Lc().T))

    @cached(['col_cov'])
    def LcCtildeLc(self, i):
        return sp.dot(self.Lc().T, sp.dot(self.Ctilde(i), self.Lc()))

    @cached(['col_cov'])
    def Cbar(self, i):
        return sp.dot(self.Wc(), sp.dot(self.Ctilde(i), self.Wc().T))

    #####################
    # Debug quantities
    #####################
    def L(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        return sp.kron(self.Lc(), sp.eye(self.dim_r))

    def W(self):
        return sp.dot(self.Wc(), self.Wr())

    def R(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        return sp.dot(self.G, self.G.T)

    #####################
    # Overwritten covar_base methods
    #####################
    @cached(['row_cov', 'col_cov', 'covar_base'])
    def K(self):
        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        rv = sp.kron(self.Cr.K(), self.R()) + sp.kron(self.Cn.K(), sp.eye(self.dim_r))
        return rv

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def K_grad_i(self,i):
        n = self.getNumberParams()

        if i >= n:
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")

        if self.dim > _MAX_DIM:
            raise TooExpensiveOperationError(msg_too_expensive_dim(my_name(),
                                                                   _MAX_DIM))

        i = self._actindex2index(i)

        nCr = self.Cr.getNumberParams()

        if i < nCr:
            rv= sp.kron(self.Cr.K_grad_i(i), self.R())
        else:
            _i = i - nCr
            rv = sp.kron(self.Cn.K_grad_i(_i), sp.eye(self.dim_r))
        return rv

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def logdet(self):
        rv = sp.sum(sp.log(self.Cn.S())) * self.dim_r
        rv+= sp.log(self.SpI()).sum()
        rv+= sp.log(self.Se()).sum() * self.rank_r
        rv+= self.logdetSg() * self.rank_c
        return rv

    @cached(['row_cov', 'col_cov', 'covar_base'])
    def logdet_grad_i(self,i):

        if i >= self.getNumberParams():
            raise ValueError("Trying to retrieve the gradient over a "
                             "parameter that is inactive.")
        i = self._actindex2index(i)

        if i < self.Cr.getNumberParams():
            trR = self.trSg()
            diagR = self.Sg()
        else:
            trR = self.dim_r
            diagR = sp.ones(self.rank_r)
        rv = self.Ctilde(i).diagonal().sum() * trR
        rv-= (self.d() * sp.kron(sp.diag(self.Cbar(i)), diagR)).sum()
        return rv



"""LMM testing code"""
import unittest
import scipy as SP
import pdb
import sys
import limix.deprecated as dlimix
from covar import Acovar_test

class CSumCF_test(unittest.TestCase,Acovar_test):
    """test class for CSumCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=10
        n_dim1=8
        n_dim2=12
        self.C=dlimix.CSumCF()
        self.C.addCovariance(dlimix.CCovSqexpARD(n_dim1));
        self.C.addCovariance(dlimix.CCovLinearARD(n_dim2));
        self.n_dim=self.C.getNumberDimensions()
        X=SP.rand(self.n,self.n_dim)
        self.C.setX(X)
        self.name = 'CSumCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CProductCF_test(unittest.TestCase,Acovar_test):
    """test class for CProductCF"""
    def setUp(self):
        SP.random.seed(1)
        self.n=10
        n_dim1=8
        n_dim2=12
        self.C=dlimix.CProductCF()
        self.C.addCovariance(dlimix.CCovSqexpARD(n_dim1));
        self.C.addCovariance(dlimix.CCovLinearARD(n_dim2));
        self.n_dim=self.C.getNumberDimensions()
        X=SP.rand(self.n,self.n_dim)
        self.C.setX(X)
        self.name = 'CProductCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CMixedX_test(unittest.TestCase,Acovar_test):
    """test class to ensure that X input free classed (CFixedCF and related) are handled propperly"""

    def setUp(self):
        SP.random.seed(1)
        self.n=10
        n_dim2=12
        K0 = SP.eye(self.n)
        self.C=dlimix.CSumCF()
        #sum of fixed CF and linearARD
        covar1 = dlimix.CFixedCF(K0)
        covar2 = dlimix.CCovLinearARD(n_dim2)
        self.C.addCovariance(covar1)
        self.C.addCovariance(covar2)
        self.n_dim=self.C.getNumberDimensions()
        self.X=SP.rand(self.n,self.n_dim)
        self.C.setX(self.X)
        self.name = 'CSumCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

    def testX_stuff(self):
        """test that X handling is consistent across covariances"""
        self.assertTrue((self.X==self.C.getX()).all())


class CKroneckerCF_test(unittest.TestCase,Acovar_test):
    """test class for CKroneckerCF"""
    def setUp(self):
        SP.random.seed(1)
        n1=3
        n2=5
        n_dim1=8
        n_dim2=12
        X1 = SP.rand(n1,n_dim1)
        X2 = SP.rand(n2,n_dim2)
        C1 = dlimix.CCovSqexpARD(n_dim1); C1.setX(X1)
        C2 = dlimix.CCovLinearARD(n_dim2);  C2.setX(X2)
        self.C = dlimix.CKroneckerCF()
        self.C.setRowCovariance(C1)
        self.C.setColCovariance(C2)
        self.n = self.C.Kdim()
        self.n_dim=self.C.getNumberDimensions()
        self.name = 'CKroneckerCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

class CKroneckerCFsoft_test(unittest.TestCase,Acovar_test):
    """test class for CKroneckerCF, when using soft indexes"""
    def setUp(self):
        SP.random.seed(1)
        nr=3
        nc=5
        n_dim1=8
        n_dim2=12
        #truncation of soft kronecker
        self.n_trunk = 10
        Xr = SP.rand(nr,n_dim1)
        Xc = SP.rand(nc,n_dim2)
        Cr = dlimix.CCovSqexpARD(n_dim1); Cr.setX(Xr)
        Cc = dlimix.CCovLinearARD(n_dim2);  Cc.setX(Xc)
        self.C = dlimix.CKroneckerCF()
        self.C.setRowCovariance(Cr)
        self.C.setColCovariance(Cc)
        #set kronecker index
        self.kronecker_index = dlimix.CKroneckerCF.createKroneckerIndex(nc,nr)
        self.n = self.C.Kdim()
        self.n_dim=self.C.getNumberDimensions()
        self.name = 'CKroneckerCF'
        self.n_params=self.C.getNumberParams()
        params=SP.exp(SP.randn(self.n_params))
        self.C.setParams(params)

    def test_kron(self):
        """test that this is a valid Kronecker"""
        self.C.setKroneckerIndicator(SP.zeros([0,0],dtype='int'))
        K1 = self.C.K()
        self.C.setKroneckerIndicator(self.kronecker_index)
        K2 = self.C.K()
        self.assertTrue((K1==K2).all())


    def test_trunk(self):
        """test whether resulting covariance function is truncated"""
        self.C.setKroneckerIndicator(self.kronecker_index[0:self.n_trunk])
        K =self.C.K()
        self.assertTrue(self.C.Kdim()==self.n_trunk)
        self.assertTrue(K.shape[0]==self.n_trunk)


if __name__ == '__main__':
    unittest.main()
